@@ -13,6 +13,7 @@
 #include <float.h>
 #include <string.h>
 #include <ctype.h>
+#include <stdarg.h>
 #include <mpi.h>
 
 #ifdef __cplusplus
@@ -75,6 +76,12 @@ extern "C" {
 #define INDEX(ix,bx,bit,WL) ix=bit/WL;bx=bit%WL /* map global column (bit)   */
                                                 /* to word (ix) and bit (bx) */
 
+/* Used in PGAEvalCompare and others */
+static inline double CMP (const double a, const double b)
+{
+    return (a < b ? -1 : (a > b ? 1 : 0));
+}
+
 /*****************************************
 *       ABSTRACT DATA TYPES              *
 *****************************************/
@@ -129,7 +136,6 @@ extern "C" {
 #define PGA_DEBUG_PRINTVAR       82
 #define PGA_DEBUG_SEND           22
 #define PGA_DEBUG_RECV           23
-#define PGA_DEBUG_MAXPGAPACKFUNCTIONS	300
 #define PGA_DEBUG_MAXFLAGS       1000
 
 /*****************************************
@@ -160,6 +166,7 @@ extern "C" {
 #define PGA_SELECT_TOURNAMENT   3    /* tournament selection                */
 #define PGA_SELECT_PTOURNAMENT  4    /* probabilistic tournament selection  */
 #define PGA_SELECT_TRUNCATION   5    /* truncation selection                */
+#define PGA_SELECT_LINEAR       6    /* linear selection                    */
 
 /*****************************************
 *            FITNESS                     *
@@ -182,14 +189,30 @@ extern "C" {
 #define PGA_MUTATION_UNIFORM    3    /* Real: +- Uniform random no.         */
 #define PGA_MUTATION_GAUSSIAN   4    /* Real: +- Gaussian random no.        */
 #define PGA_MUTATION_PERMUTE    5    /* Integer: Permutation (swap)         */
+#define PGA_MUTATION_DE         6    /* Differential Evolution (only real)  */
+
+/*****************************************
+* Differential Evolution Variant         *
+*****************************************/
+#define PGA_DE_VARIANT_RAND      1   /* Standard DE */
+#define PGA_DE_VARIANT_BEST      2   /* Derive from best string */
+#define PGA_DE_VARIANT_EITHER_OR 3   /* Either-or variant */
+
+/*******************************************
+* Differential Evolution Crossover Variant *
+*******************************************/
+#define PGA_DE_CROSSOVER_BIN      1  /* Standard DE binomial crossover */
+#define PGA_DE_CROSSOVER_EXP      2  /* Exponential crossover */
 
 /*****************************************
 *        POPULATION REPLACEMENT          *
 *****************************************/
-#define PGA_POPREPL_BEST         1   /* Select best   string                */
-#define PGA_POPREPL_RANDOM_NOREP 2   /* Select random string w/o replacement*/
-#define PGA_POPREPL_RANDOM_REP   3   /* Select random string w/  replacement*/
-#define PGA_POPREPL_RTR          4   /* Restricted tournament replacement   */
+#define PGA_POPREPL_BEST          1  /* Select best   string                */
+#define PGA_POPREPL_RANDOM_NOREP  2  /* Select random string w/o replacement*/
+#define PGA_POPREPL_RANDOM_REP    3  /* Select random string w/  replacement*/
+#define PGA_POPREPL_RTR           4  /* Restricted tournament replacement   */
+#define PGA_POPREPL_PAIRWISE_BEST 5  /* Pairwise compare old/newpop         */
+#define PGA_POPREPL_NSGA_II       6  /* NSGA-II non-dominated sorting       */
 
 /****************************************
  *       REPORT OPTIONS                 *
@@ -226,7 +249,8 @@ extern "C" {
 #define PGA_USERFUNCTION_STOPCOND                9
 #define PGA_USERFUNCTION_ENDOFGEN                10
 #define PGA_USERFUNCTION_GEN_DIFFERENCE          11
-#define PGA_NUM_USERFUNCTIONS                    11
+#define PGA_USERFUNCTION_PRE_EVAL                12
+#define PGA_NUM_USERFUNCTIONS                    12
 
 /*****************************************
 *           MPI SEND/RECV TAGS           *
@@ -239,14 +263,23 @@ extern "C" {
 
 
 /*****************************************
-*       INDIVIDUAL STRUTURE              *
+*       INDIVIDUAL STRUCTURE             *
 *****************************************/
 
-typedef struct {                    /* primary population data structure   */
-  double evalfunc;                  /* evaluation function value           */
-  double fitness;                   /* fitness    function value           */
-  int    evaluptodate;              /* flag whether evalfunc is current    */
-  void   *chrom;                    /* pointer to the GA string            */
+typedef struct PGAIndividual {         /* primary population data structure */
+  double                evalue;        /* evaluation function value         */
+  double                fitness;       /* fitness    function value         */
+  int                   evaluptodate;  /* flag whether evalue is current    */
+  void                 *chrom;         /* pointer to the GA string          */
+  double               *auxeval;       /* Auxiliary evaluations             */
+  double                auxtotal;      /* Total aux evaluation              */
+  int                   auxtotalok;    /* flag wether auxtotal is current*/
+  unsigned int          rank;          /* Rank for dominance-sorting        */
+  /* The following are not transmitted via MPI */
+  struct PGAContext    *ctx;           /* Pointer to our PGAContext         */
+  struct PGAIndividual *pop;           /* The population of this indiv.     */
+  double                crowding;      /* Crowding metric for NSGA-II       */
+  int                   funcidx;       /* Temporary function index          */
 } PGAIndividual;
 
 
@@ -260,6 +293,10 @@ typedef struct {
     int fw;                  /* number of full (WL length) words          */
     int eb;                  /* number of extra bits in last NOT full word*/
     int PopSize;             /* Number of strings to use                  */
+    int NumAuxEval;          /* Number of auxiliary evaluation values     */
+    int NumConstraint;       /* Number of constraints                     */
+    int SumConstraints;      /* PGA_TRUE if no dominance-sorting for
+                                constraints                               */
     int StringLen;           /* string lengths                            */
     int StoppingRule;        /* Termination Criteria                      */
     int MaxIter;             /* Maximum number of iterations to run       */
@@ -276,14 +313,26 @@ typedef struct {
     int SelectIndex;         /* index of Select for next two individuals  */
     int FitnessType;         /* Type of fitness transformation used       */
     int FitnessMinType;      /* Transformation for minimization problems  */
+    int MutateOnly;          /* Mutate only, no crossover at all          */
     int MutateOnlyNoCross;   /* Mutate only strings not from crossover    */
     int MutationType;        /* Type of mutation used                     */
     int MutateIntegerValue;  /* Multiplier to mutate Integer strings with */
-    int MutateBoundedFlag;   /* Confine integer alleles to given range    */
+    int MutateBoundedFlag;   /* Confine alleles to given range (bound)    */
+    int MutateBounceFlag;    /* Confine alleles to given range (random)   */
     int TournamentSize;      /* Number of participants in tournament      */
     int RTRWindowSize;       /* Window for restricted tournament select   */
     int TournamentWithRepl;  /* Tournament with / without replacement     */
     int RandomizeSelect;     /* Additional randomisation during select    */
+    int DEVariant;           /* Differential evolution (DE) variant       */
+    int DENumDiffs;          /* Number of differences for DE (1 or 2)     */
+    int DECrossoverType;     /* Crossover type DE                         */
+    int DEDitherPerIndividual; /* Per indidivual or per generation        */
+    double DEDither;         /* Dither value centered around F            */
+    double DEScaleFactor;    /* Scale Factor F for DE                     */
+    double DEAuxFactor;      /* Auxiliary Factor K for DE                 */
+    double DECrossoverProb;  /* Crossover probability Cr for DE           */
+    double DEJitter;         /* Jitter interval DE (uniform dist)         */
+    double DEProbabilityEO;  /* Either-Or-Probability for DE              */
     double MutateRealValue;  /* Multiplier to mutate Real strings with    */
     double MutationProb;     /* Starting mutation probability             */
     double CrossoverProb;    /* Crossover probability                     */
@@ -319,6 +368,8 @@ typedef struct {
     int          (*StopCond)(PGAContext *);
     void         (*EndOfGen)(PGAContext *);
     double       (*GeneDistance)(PGAContext *, int, int, int, int);
+    void         (*PreEval)(PGAContext *, int);
+    int          (*EvalCompare)(PGAContext *, int, int, int, int);
 } PGACOperations;
 
 typedef struct {
@@ -331,6 +382,8 @@ typedef struct {
     int          (*StopCond)(void *);
     void         (*EndOfGen)(void *);
     double       (*GeneDistance)(void *, void *, void *, void *, void *);
+    void         (*PreEval)(void *, void *);
+    int          (*EvalCompare)(void *, void *, void *, void *, void *);
 } PGAFortranOperations;
 
 typedef struct sample_state_s {
@@ -356,13 +409,18 @@ typedef struct {
 *          REPORT STRUCTURE              *
 *****************************************/
 typedef struct {
-     int    PrintFreq;               /* How often to print statistics reports*/
-     int    PrintOptions;
-     double Offline;
-     double Online;
-     double Average;
-     double Best;
-     time_t starttime;
+     int             PrintFreq;    /* How often to print statistics reports */
+     int             PrintOptions;
+     double         *Offline;      /* One value for each function           */
+     double         *Online;       /* One value for each function           */
+     double         *Average;      /* One value for each function           */
+     double         *Best;         /* One value for each function           */
+     int            *BestIdx;      /* Indices of best individuals           */
+     int             validcount;   /* # indivs w/o constraint-violations    */
+     int             validonline;  /* cumulated validcount                  */
+     int             validoffline; /* cumulated best indiv valid count      */
+     int             nevals;       /* Number of evaluations                 */
+     time_t          starttime;
 } PGAReport;
 
 
@@ -406,8 +464,9 @@ typedef struct {
 *      SCRATCH DATA STRUCTURES           *
 *****************************************/
 typedef struct {
-    int    *intscratch;            /* integer-scratch space                 */
-    double *dblscratch;            /* double- scratch space                 */
+    int          *intscratch;            /* integer-scratch space          */
+    double       *dblscratch;            /* double- scratch space          */
+    unsigned int *dominance;             /* for dominance sorting          */
 } PGAScratch;
 
 /*****************************************
@@ -494,6 +553,12 @@ void PGASetRandomInitFlag(PGAContext *ctx, int RandomBoolean);
 int PGAGetRandomInitFlag (PGAContext *ctx);
 void PGACreatePop (PGAContext *ctx, int pop);
 void PGACreateIndividual (PGAContext *ctx, int p, int pop, int initflag);
+void PGASetNumAuxEval (PGAContext *ctx, int n);
+int PGAGetNumAuxEval (PGAContext *ctx);
+void PGASetNumConstraint (PGAContext *ctx, int n);
+int PGAGetNumConstraint (PGAContext *ctx);
+void PGASetSumConstraintsFlag (PGAContext *ctx, int n);
+int PGAGetSumConstraintsFlag (PGAContext *ctx);
 
 /*****************************************
 *          cross.c
@@ -559,50 +624,65 @@ int PGAGetNoDuplicatesFlag (PGAContext *ctx);
 *          evaluate.c
 *****************************************/
 
-void PGASetEvaluation ( PGAContext *ctx, int p, int pop, double val );
-double PGAGetEvaluation ( PGAContext *ctx, int p, int pop );
-void PGASetEvaluationUpToDateFlag ( PGAContext *ctx, int p, int pop,
-                                   int status );
-int PGAGetEvaluationUpToDateFlag ( PGAContext *ctx, int p, int pop );
-double PGAGetRealFromBinary(PGAContext *ctx, int p, int pop, int start,
-                            int end, double lower, double upper);
-double PGAGetRealFromGrayCode(PGAContext *ctx, int p, int pop, int start,
-                                  int end, double lower, double upper);
-void PGAEncodeRealAsBinary(PGAContext *ctx, int p, int pop, int start,
-                               int end, double low, double high, double val);
-void PGAEncodeRealAsGrayCode(PGAContext *ctx, int p, int pop, int start,
+/* Macro trickery for optional aux argument */
+/* See https://stackoverflow.com/questions/1472138/c-default-arguments */
+struct opt_ptr_ptr { const double **d; };
+struct opt_ptr     { const double *d;  };
+void _PGASetEvaluation
+    (PGAContext *ctx, int p, int pop, double v, const double *aux);
+static inline void _PGASetEvaluation_s
+    (PGAContext *ctx, int p, int pop, double v, const struct opt_ptr opt_ptr)
+{
+    _PGASetEvaluation (ctx, p, pop, v, opt_ptr.d);
+}
+#define PGASetEvaluation(a, b, c, d, ...) \
+    _PGASetEvaluation_s(a, b, c, d, (struct opt_ptr){__VA_ARGS__})
+double _PGAGetEvaluation (PGAContext *ctx, int p, int pop, const double **aux);
+static inline double _PGAGetEvaluation_s
+    (PGAContext *ctx, int p, int pop, const struct opt_ptr_ptr opt_ptr_ptr)
+{
+    return _PGAGetEvaluation (ctx, p, pop, opt_ptr_ptr.d);
+}
+#define PGAGetEvaluation(a, b, c, ...) \
+    _PGAGetEvaluation_s(a, b, c, (struct opt_ptr_ptr){__VA_ARGS__})
+double *PGAGetAuxEvaluation (PGAContext *ctx, int p, int pop);
+void PGASetEvaluationUpToDateFlag (PGAContext *ctx, int p, int pop, int status);
+int PGAGetEvaluationUpToDateFlag (PGAContext *ctx, int p, int pop);
+double PGAGetRealFromBinary (PGAContext *ctx, int p, int pop, int start,
+                             int end, double lower, double upper);
+double PGAGetRealFromGrayCode (PGAContext *ctx, int p, int pop, int start,
+                               int end, double lower, double upper);
+void PGAEncodeRealAsBinary (PGAContext *ctx, int p, int pop, int start,
+                            int end, double low, double high, double val);
+void PGAEncodeRealAsGrayCode (PGAContext *ctx, int p, int pop, int start,
                               int end, double low, double high, double val);
-int PGAGetIntegerFromBinary(PGAContext *ctx, int p, int pop, int start,
+int PGAGetIntegerFromBinary (PGAContext *ctx, int p, int pop, int start,
                                  int end);
-int PGAGetIntegerFromGrayCode(PGAContext *ctx, int p, int pop, int start,
-                                   int end);
-void PGAEncodeIntegerAsBinary(PGAContext *ctx, int p, int pop, int start,
-                              int end, int val);
-void PGAEncodeIntegerAsGrayCode(PGAContext *ctx, int p, int pop, int start,
-                                int end, int val);
+int PGAGetIntegerFromGrayCode (PGAContext *ctx, int p, int pop, int start,
+                               int end);
+void PGAEncodeIntegerAsBinary (PGAContext *ctx, int p, int pop, int start,
+                               int end, int val);
+void PGAEncodeIntegerAsGrayCode (PGAContext *ctx, int p, int pop, int start,
+                                 int end, int val);
 double PGAMapIntegerToReal (PGAContext *ctx, int v, int a, int b, double l,
                             double u);
-int PGAMapRealToInteger(PGAContext *ctx, double r, double l, double u, int a,
-                        int b);
+int PGAMapRealToInteger (PGAContext *ctx, double r, double l, double u, int a,
+                         int b);
 
 /*****************************************
 *          fitness.c
 *****************************************/
 
-void PGAFitness ( PGAContext *ctx, int popindex );
-int PGARank( PGAContext *ctx, int p, int *order, int n );
-double PGAGetFitness ( PGAContext *ctx, int p, int pop );
+void PGAFitness (PGAContext *ctx, int popindex);
+int PGARank (PGAContext *ctx, int p, int *order, int n);
+double PGAGetFitness (PGAContext *ctx, int p, int pop);
 int PGAGetFitnessType (PGAContext *ctx);
 int PGAGetFitnessMinType (PGAContext *ctx);
 double PGAGetMaxFitnessRank (PGAContext *ctx);
-void PGASetFitnessType( PGAContext *ctx, int fitness_type);
-void PGASetFitnessMinType( PGAContext *ctx, int fitness_type);
-void PGASetMaxFitnessRank( PGAContext *ctx, double fitness_rank_max);
-void PGAFitnessLinearNormal ( PGAContext *ctx, PGAIndividual *pop );
-void PGAFitnessLinearRank ( PGAContext *ctx, PGAIndividual *pop );
-void PGAFitnessMinReciprocal ( PGAContext *ctx, PGAIndividual *pop );
-void PGAFitnessMinCmax ( PGAContext *ctx, PGAIndividual *pop );
-void PGASetFitnessCmaxValue( PGAContext *ctx, double val);
+void PGASetFitnessType (PGAContext *ctx, int fitness_type);
+void PGASetFitnessMinType (PGAContext *ctx, int fitness_type);
+void PGASetMaxFitnessRank (PGAContext *ctx, double fitness_rank_max);
+void PGASetFitnessCmaxValue (PGAContext *ctx, double val);
 double PGAGetFitnessCmaxValue (PGAContext *ctx);
 
 /*****************************************
@@ -611,13 +691,6 @@ double PGAGetFitnessCmaxValue (PGAContext *ctx);
 
 double PGAHammingDistance( PGAContext *ctx, int popindex);
 
-/*****************************************
-*          heap.c
-*****************************************/
-
-void PGADblHeapSort ( PGAContext *ctx, double *a, int *idx, int n );
-void PGAIntHeapSort ( PGAContext *ctx, int *a, int *idx, int n );
-
 /*****************************************
 *          integer.c
 *****************************************/
@@ -625,7 +698,7 @@ void PGAIntHeapSort ( PGAContext *ctx, int *a, int *idx, int n );
 void PGASetIntegerAllele (PGAContext *ctx, int p, int pop, int i, int value);
 int PGAGetIntegerAllele (PGAContext *ctx, int p, int pop, int i);
 void PGASetIntegerInitPermute ( PGAContext *ctx, int min, int max);
-void PGASetIntegerInitRange (PGAContext *ctx, int *min, int *max);
+void PGASetIntegerInitRange (PGAContext *ctx, const int *min, const int *max);
 int PGAGetIntegerInitType (PGAContext *ctx);
 int PGAGetMinIntegerInitValue (PGAContext *ctx, int i);
 int PGAGetMaxIntegerInitValue (PGAContext *ctx, int i);
@@ -644,6 +717,8 @@ void PGAIntegerInitString(PGAContext *ctx, int p, int pop);
 MPI_Datatype PGAIntegerBuildDatatype(PGAContext *ctx, int p, int pop);
 double PGAIntegerGeneDistance (PGAContext *ctx, int p1, int pop1, int p2,
                                int pop2);
+double PGAIntegerEuclidianDistance (PGAContext *ctx, int p1, int pop1, int p2,
+                              int pop2);
 
 /*****************************************
 *          mpi_stub.c
@@ -663,25 +738,50 @@ void PGASetMutationIntegerValue( PGAContext *ctx, int val);
 int PGAGetMutationIntegerValue (PGAContext *ctx);
 void PGASetMutationBoundedFlag(PGAContext *ctx, int val);
 int PGAGetMutationBoundedFlag(PGAContext *ctx);
+void PGASetMutationBounceBackFlag(PGAContext *ctx, int val);
+int PGAGetMutationBounceBackFlag(PGAContext *ctx);
 void PGASetMutationProb(PGAContext *ctx, double mutation_prob);
 double PGAGetMutationProb (PGAContext *ctx);
+void PGASetDEVariant (PGAContext *ctx, int val);
+int PGAGetDEVariant (PGAContext *ctx);
+void PGASetDENumDiffs (PGAContext *ctx, int val);
+int PGAGetDENumDiffs (PGAContext *ctx);
+void PGASetDEScaleFactor (PGAContext *ctx, double val);
+double PGAGetDEScaleFactor (PGAContext *ctx);
+void PGASetDEAuxFactor (PGAContext *ctx, double val);
+double PGAGetDEAuxFactor (PGAContext *ctx);
+void PGASetDECrossoverProb (PGAContext *ctx, double val);
+double PGAGetDECrossoverProb (PGAContext *ctx);
+void PGASetDEJitter (PGAContext *ctx, double val);
+double PGAGetDEJitter (PGAContext *ctx);
+void PGASetDEProbabilityEO (PGAContext *ctx, double val);
+double PGAGetDEProbabilityEO (PGAContext *ctx);
+void PGASetDECrossoverType (PGAContext *ctx, int val);
+int PGAGetDECrossoverType (PGAContext *ctx);
+void PGASetDEDither (PGAContext *ctx, double val);
+double PGAGetDEDither (PGAContext *ctx);
+void PGASetDEDitherPerIndividual (PGAContext *ctx, int val);
+int PGAGetDEDitherPerIndividual (PGAContext *ctx);
 
 /*****************************************
 *          parallel.c
 *****************************************/
 
-void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int),
+void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int, double *),
 	      MPI_Comm comm);
 void PGAEvaluateSeq(PGAContext *ctx, int pop,
-		    double (*f)(PGAContext *, int, int));
+		    double (*f)(PGAContext *, int, int, double *));
 void PGAEvaluateCoop(PGAContext *ctx, int pop,
-		     double (*f)(PGAContext *, int, int), MPI_Comm comm);
+		     double (*f)(PGAContext *, int, int, double *),
+                     MPI_Comm comm);
 void PGAEvaluateMS(PGAContext *ctx, int pop,
-		   double (*f)(PGAContext *c, int p, int pop), MPI_Comm comm);
+		   double (*f)(PGAContext *c, int p, int pop, double *),
+                   MPI_Comm comm);
 void PGAEvaluateSlave(PGAContext *ctx, int pop,
-		      double (*f)(PGAContext *, int, int), MPI_Comm comm);
+		      double (*f)(PGAContext *, int, int, double *),
+                      MPI_Comm comm);
 void PGAEvaluate(PGAContext *ctx, int pop,
-		 double (*f)(PGAContext *, int, int), MPI_Comm comm);
+		 double (*f)(PGAContext *, int, int, double *), MPI_Comm comm);
 MPI_Datatype PGABuildDatatype(PGAContext *ctx, int p, int pop);
 void PGASendIndividual(PGAContext *ctx, int p, int pop, int dest, int tag,
                        MPI_Comm comm);
@@ -691,9 +791,10 @@ void PGASendReceiveIndividual(PGAContext *ctx, int send_p, int send_pop,
                               int dest, int send_tag, int recv_p, int recv_pop,
                               int source, int recv_tag, MPI_Comm comm,
                               MPI_Status *status);
-void PGARunIM(PGAContext *ctx, double (*f)(PGAContext *c, int p, int pop),
-              MPI_Comm tcomm);
-void PGARunNM(PGAContext *ctx, double (*f)(PGAContext *c, int p, int pop),
+void PGARunIM(PGAContext *ctx, double (*f)
+              (PGAContext *c, int p, int pop, double *), MPI_Comm tcomm);
+void PGARunNM(PGAContext *ctx,
+              double (*f)(PGAContext *c, int p, int pop, double *),
               MPI_Comm tcomm);
 int PGAGetRank (PGAContext *ctx, MPI_Comm comm);
 int PGAGetNumProcs (PGAContext *ctx, MPI_Comm comm);
@@ -708,20 +809,24 @@ MPI_Comm PGAGetCommunicator( PGAContext *ctx);
 *          pga.c
 *****************************************/
 
-void PGARun(PGAContext *ctx, double (*evaluate)(PGAContext *c, int p, int pop));
+void PGARun(PGAContext *ctx,
+    double (*evaluate)(PGAContext *c, int p, int pop, double *auxeval));
 void PGARunMutationAndCrossover (PGAContext *ctx, int oldpop, int newpop);
 void PGARunMutationOrCrossover ( PGAContext *ctx, int oldpop, int newpop );
+void PGARunMutationOnly ( PGAContext *ctx, int oldpop, int newpop );
 void PGAUpdateGeneration(PGAContext *ctx, MPI_Comm comm);
 int PGAGetDataType (PGAContext *ctx);
 int PGAGetOptDirFlag (PGAContext *ctx);
 int PGAGetStringLength (PGAContext *ctx);
 int PGAGetVariableStringLength (PGAContext *ctx, int p, int pop);
 int PGAGetGAIterValue (PGAContext *ctx);
+int PGAGetEvalCount (PGAContext *ctx);
 void PGASetMutationOrCrossoverFlag( PGAContext *ctx, int flag);
 void PGASetMutationAndCrossoverFlag( PGAContext *ctx, int flag);
+void PGASetMutationOnlyFlag( PGAContext *ctx, int flag);
 int PGAGetMutationOrCrossoverFlag (PGAContext *ctx);
 int PGAGetMutationAndCrossoverFlag (PGAContext *ctx);
-void PGARestrictedTournamentReplacement (PGAContext *ctx);
+int PGAGetMutationOnlyFlag (PGAContext *ctx);
 
 /*****************************************
 *          pop.c
@@ -737,6 +842,9 @@ void PGASetPopSize (PGAContext *ctx, int popsize);
 void PGASetNumReplaceValue( PGAContext *ctx, int pop_replace);
 void PGASetPopReplaceType( PGAContext *ctx, int pop_replace);
 void PGASetRTRWindowSize (PGAContext *ctx, int window);
+void PGARestrictedTournamentReplacement (PGAContext *ctx);
+void PGAPairwiseBestReplacement (PGAContext *ctx);
+void PGA_NSGA_II_Replacement (PGAContext *ctx);
 
 /*****************************************
 *          random.c
@@ -759,7 +867,7 @@ int PGARandomNextSample(PGASampleState *state);
 void PGASetRealAllele (PGAContext *ctx, int p, int pop, int i, double value);
 double PGAGetRealAllele (PGAContext *ctx, int p, int pop, int i);
 void PGASetRealInitPercent ( PGAContext *ctx, double *median, double *percent);
-void PGASetRealInitRange (PGAContext *ctx, double *min, double *max);
+void PGASetRealInitRange (PGAContext *ctx, const double *min, const double *max);
 double PGAGetMinRealInitValue (PGAContext *ctx, int i);
 double PGAGetMaxRealInitValue (PGAContext *ctx, int i);
 int PGAGetRealInitType (PGAContext *ctx);
@@ -778,6 +886,8 @@ void PGARealInitString ( PGAContext *ctx, int p, int pop);
 MPI_Datatype PGARealBuildDatatype(PGAContext *ctx, int p, int pop);
 double PGARealGeneDistance (PGAContext *ctx, int p1, int pop1, int p2,
                             int pop2);
+double PGARealEuclidianDistance (PGAContext *ctx, int p1, int pop1, int p2,
+                              int pop2);
 
 /*****************************************
 *          report.c
@@ -808,25 +918,28 @@ double PGAGetRestartAlleleChangeProb(PGAContext *ctx);
 *          select.c
 *****************************************/
 
-void PGASelect( PGAContext *ctx, int popix );
-int PGASelectNextIndex ( PGAContext *ctx, int popix );
-void PGASetSelectType( PGAContext *ctx, int select_type);
+void PGASelect (PGAContext *ctx, int popix);
+int PGASelectNextIndex (PGAContext *ctx, int popix);
+void PGASetSelectType (PGAContext *ctx, int select_type);
 int PGAGetSelectType (PGAContext *ctx);
-void PGASetPTournamentProb(PGAContext *ctx, double ptournament_prob);
-double PGAGetPTournamentProb(PGAContext *ctx);
-int PGASelectProportional(PGAContext *ctx, PGAIndividual *pop);
-void PGASelectSUS( PGAContext *ctx, PGAIndividual *pop );
-int PGASelectTournament( PGAContext *ctx, PGAIndividual *pop );
-int PGASelectPTournament( PGAContext *ctx, PGAIndividual *pop );
-int PGASelectTruncation( PGAContext *ctx, PGAIndividual *pop );
-void PGASetTournamentSize(PGAContext *ctx, int tournament_size);
-int PGAGetTournamentSize(PGAContext *ctx);
-void PGASetTournamentWithReplacement(PGAContext *ctx, int value);
-int PGAGetTournamentWithReplacement(PGAContext *ctx);
-void PGASetTruncationProportion(PGAContext *ctx, double proportion);
-double PGAGetTruncationProportion(PGAContext *ctx);
-void PGASetRandomizeSelect(PGAContext *ctx, int value);
-int PGAGetRandomizeSelect(PGAContext *ctx);
+void PGASetPTournamentProb (PGAContext *ctx, double ptournament_prob);
+double PGAGetPTournamentProb (PGAContext *ctx);
+int PGASelectProportional (PGAContext *ctx, PGAIndividual *pop);
+void PGASelectSUS (PGAContext *ctx, PGAIndividual *pop);
+int PGASelectTournament (PGAContext *ctx, int pop);
+int PGASelectPTournament (PGAContext *ctx, int pop);
+int PGASelectTruncation (PGAContext *ctx, int pop);
+int PGASelectLinear (PGAContext *ctx, PGAIndividual *pop);
+void PGASetTournamentSize (PGAContext *ctx, int tournament_size);
+int PGAGetTournamentSize (PGAContext *ctx);
+void PGASetTournamentWithReplacement (PGAContext *ctx, int value);
+int PGAGetTournamentWithReplacement (PGAContext *ctx);
+void PGASetTruncationProportion (PGAContext *ctx, double proportion);
+double PGAGetTruncationProportion (PGAContext *ctx);
+void PGASetRandomizeSelect (PGAContext *ctx, int value);
+int PGAGetRandomizeSelect (PGAContext *ctx);
+double INDGetAuxTotal (PGAIndividual *ind);
+double PGAGetAuxTotal (PGAContext *ctx, int p, int pop);
 
 /*****************************************
 *          stop.c
@@ -845,15 +958,16 @@ void PGASetMaxSimilarityValue(PGAContext *ctx, int max_similarity);
 *          system.c
 *****************************************/
 
-void PGAError( PGAContext *ctx, char *msg,
-               int level, int datatype, void *data );
+void PGAError (PGAContext *ctx, char *msg,
+               int level, int datatype, void *data);
+void PGAErrorPrintf (PGAContext *ctx, int level, char *fmt, ...);
 void PGADestroy (PGAContext *ctx);
 int PGAGetMaxMachineIntValue (PGAContext *ctx);
 int PGAGetMinMachineIntValue (PGAContext *ctx);
 double PGAGetMaxMachineDoubleValue (PGAContext *ctx);
 double PGAGetMinMachineDoubleValue (PGAContext *ctx);
-void PGAUsage( PGAContext *ctx );
-void PGAPrintVersionNumber( PGAContext *ctx );
+void PGAUsage (PGAContext *ctx);
+void PGAPrintVersionNumber (PGAContext *ctx);
 
 /*****************************************
 *          user.c
@@ -865,18 +979,26 @@ void PGASetUserFunction(PGAContext *ctx, int constant, void *f);
 *          utility.c
 *****************************************/
 
-double PGAMean ( PGAContext *ctx, double *a, int n);
-double PGAStddev ( PGAContext *ctx, double *a, int n, double mean);
-int PGARound(PGAContext *ctx, double x);
-void PGACopyIndividual( PGAContext *ctx, int p1, int pop1, int p2, int pop2);
-int PGACheckSum(PGAContext *ctx, int p, int pop);
-int PGAGetWorstIndex(PGAContext *ctx, int pop);
-int PGAGetBestIndex(PGAContext *ctx, int pop);
-PGAIndividual *PGAGetIndividual ( PGAContext *ctx, int p, int pop);
-void PGAUpdateAverage(PGAContext *ctx, int pop);
-void PGAUpdateOnline(PGAContext *ctx, int pop);
-void PGAUpdateOffline(PGAContext *ctx, int pop);
-int PGAComputeSimilarity(PGAContext *ctx, PGAIndividual *pop);
+double PGAMean (PGAContext *ctx, double *a, int n);
+double PGAStddev (PGAContext *ctx, double *a, int n, double mean);
+int PGARound (PGAContext *ctx, double x);
+void INDCopyIndividual (PGAIndividual *src, PGAIndividual *dst);
+void PGACopyIndividual (PGAContext *ctx, int p1, int pop1, int p2, int pop2);
+int PGACheckSum (PGAContext *ctx, int p, int pop);
+int PGAGetWorstIndex (PGAContext *ctx, int pop);
+int PGAGetBestIndex (PGAContext *ctx, int pop);
+double PGAGetBestReport (PGAContext *ctx, int pop, int idx);
+int PGAGetBestReportIndex (PGAContext *ctx, int pop, int idx);
+PGAIndividual *PGAGetIndividual (PGAContext *ctx, int p, int pop);
+void PGAUpdateAverage (PGAContext *ctx, int pop);
+void PGAUpdateBest (PGAContext *ctx, int pop);
+void PGAUpdateOnline (PGAContext *ctx, int pop);
+void PGAUpdateOffline (PGAContext *ctx, int pop);
+int PGAComputeSimilarity (PGAContext *ctx, int popix);
+int INDEvalCompare (PGAIndividual *ind1, PGAIndividual *ind2);
+int PGAEvalCompare (PGAContext *ctx, int p1, int pop1, int p2, int pop2);
+void PGAEvalSort (PGAContext *ctx, int pop, int *idx);
+int PGAEvalSortHelper (const void *i1, const void *i2);
 
 #ifdef __cplusplus
 }
