@@ -590,25 +590,33 @@ void PGABinaryUniformCrossover(PGAContext *ctx, int p1, int p2, int pop1,
       PGABinaryPrintString( ctx, stdout, s, PGA_NEWPOP );
 
 ****************************************************************************I*/
-void PGABinaryPrintString( PGAContext *ctx, FILE *fp, int p, int pop )
+void PGABinaryPrintString (PGAContext *ctx, FILE *fp, int p, int pop)
 {
-     PGABinary *c = (PGABinary *)PGAGetIndividual(ctx, p, pop)->chrom;
-     int i;
+    PGABinary *c = (PGABinary *)PGAGetIndividual (ctx, p, pop)->chrom;
+    int wsize = 64 / WL;
+    int i, j;
+    int done = 0;
 
-     PGADebugEntered("PGABinaryPrintString");
+    PGADebugEntered ("PGABinaryPrintString");
 
-     for( i=0; i<ctx->ga.fw; i++ ) {
-          fprintf(fp,"[ ");
-          PGABinaryPrint( ctx, fp, (c+i), WL );
-          fprintf(fp," ]\n");
-     }
-     if ( ctx->ga.eb > 0 ) {
-          fprintf(fp,"[ ");
-          PGABinaryPrint( ctx, fp, (c+ctx->ga.fw), ctx->ga.eb );
-          fprintf(fp," ]");
-     }
+    for (i=0; i<ctx->ga.fw; i+=wsize) {
+         fprintf (fp,"[ ");
+         for (j=0; j<wsize && i+j<ctx->ga.fw; j++) {
+             PGABinaryPrint (ctx, fp, (c+i+j), WL);
+         }
+         if (j < wsize && ctx->ga.eb > 0) {
+             PGABinaryPrint (ctx, fp, (c+ctx->ga.fw), ctx->ga.eb);
+             done = 1;
+         }
+         fprintf (fp," ]\n");
+    }
+    if (ctx->ga.eb > 0 && !done) {
+         fprintf (fp,"[ ");
+         PGABinaryPrint (ctx, fp, (c+ctx->ga.fw), ctx->ga.eb);
+         fprintf (fp," ]\n");
+    }
 
-     PGADebugExited("PGABinaryPrintString");
+    PGADebugExited ("PGABinaryPrintString");
 }
 
 /*I****************************************************************************
@@ -674,19 +682,21 @@ void PGABinaryCopyString (PGAContext *ctx, int p1, int pop1, int p2, int pop2)
 ****************************************************************************I*/
 int PGABinaryDuplicate( PGAContext *ctx, int p1, int pop1, int p2, int pop2)
 {
-     PGABinary *a = (PGABinary *)PGAGetIndividual(ctx, p1, pop1)->chrom;
-     PGABinary *b = (PGABinary *)PGAGetIndividual(ctx, p2, pop2)->chrom;
+     PGABinary *a = (PGABinary *)PGAGetIndividual (ctx, p1, pop1)->chrom;
+     PGABinary *b = (PGABinary *)PGAGetIndividual (ctx, p2, pop2)->chrom;
      int wi;
 
-     PGADebugEntered("PGABinaryDuplicate");
+     PGADebugEntered ("PGABinaryDuplicate");
 
-     wi = ctx->ga.tw-1;
-     if (a[0] == b[0])
-         for (; (wi>0) && (a[wi] == b[wi]); wi--);
+     for (wi=0; wi<ctx->ga.tw; wi++) {
+        if (a [wi] != b [wi]) {
+            break;
+        }
+     }
 
-     PGADebugExited("PGABinaryDuplicate");
+     PGADebugExited ("PGABinaryDuplicate");
 
-     return((wi==0) ? PGA_TRUE : PGA_FALSE);
+     return wi == ctx->ga.tw ? PGA_TRUE : PGA_FALSE;
 }
 
 /*I****************************************************************************
@@ -746,41 +756,56 @@ void PGABinaryInitString(PGAContext *ctx, int p, int pop)
 ****************************************************************************I*/
 MPI_Datatype PGABinaryBuildDatatype(PGAContext *ctx, int p, int pop)
 {
+    int            n = 6;
+    int            counts[7];      /* Number of elements in each
+                                      block (array of integer) */
+    MPI_Aint       displs[7];      /* byte displacement of each
+                                      block (array of integer) */
+    MPI_Datatype   types[7];       /* type of elements in each block (array
+                                      of handles to datatype objects) */
+    MPI_Datatype   individualtype; /* new datatype (handle) */
+    PGAIndividual *traveller;      /* address of individual in question */
 
-     int            counts[4];      /* Number of elements in each
-                                       block (array of integer) */
-     MPI_Aint       displs[4];      /* byte displacement of each
-                                       block (array of integer) */
-     MPI_Datatype   types[4];       /* type of elements in each block (array
-                                       of handles to datatype objects) */
-     MPI_Datatype   individualtype; /* new datatype (handle) */
-     PGAIndividual *traveller;      /* address of individual in question */
+    PGADebugEntered("PGABinaryBuildDatatype");
 
-     PGADebugEntered("PGABinaryBuildDatatype");
+    traveller = PGAGetIndividual(ctx, p, pop);
+    MPI_Get_address(&traveller->evalue, &displs[0]);
+    counts[0] = 1;
+    types[0]  = MPI_DOUBLE;
 
-     traveller = PGAGetIndividual(ctx, p, pop);
-     MPI_Get_address(&traveller->evalfunc, &displs[0]);
-     counts[0] = 1;
-     types[0]  = MPI_DOUBLE;
+    MPI_Get_address(&traveller->fitness, &displs[1]);
+    counts[1] = 1;
+    types[1]  = MPI_DOUBLE;
 
-     MPI_Get_address(&traveller->fitness, &displs[1]);
-     counts[1] = 1;
-     types[1]  = MPI_DOUBLE;
+    MPI_Get_address(&traveller->evaluptodate, &displs[2]);
+    counts[2] = 1;
+    types[2]  = MPI_INT;
 
-     MPI_Get_address(&traveller->evaluptodate, &displs[2]);
-     counts[2] = 1;
-     types[2]  = MPI_INT;
+    MPI_Get_address(traveller->chrom, &displs[3]);
+    counts[3] = ctx->ga.tw;
+    types[3]  = MPI_UNSIGNED_LONG;
 
-     MPI_Get_address(traveller->chrom, &displs[3]);
-     counts[3] = ctx->ga.tw;
-     types[3]  = MPI_UNSIGNED_LONG;
+    MPI_Get_address(&traveller->auxtotal, &displs[4]);
+    counts[4] = 1;
+    types[4]  = MPI_DOUBLE;
 
-     MPI_Type_create_struct(4, counts, displs, types, &individualtype);
-     MPI_Type_commit(&individualtype);
+    MPI_Get_address(&traveller->auxtotalok, &displs[5]);
+    counts[5] = 1;
+    types[5]  = MPI_INT;
 
-     PGADebugExited("PGABinaryBuildDatatype");
+    if (ctx->ga.NumAuxEval) {
+        MPI_Get_address(traveller->auxeval, &displs[6]);
+        counts[6] = ctx->ga.NumAuxEval;
+        types[6]  = MPI_DOUBLE;
+        n += 1;
+    }
 
-     return (individualtype);
+    MPI_Type_create_struct(n, counts, displs, types, &individualtype);
+    MPI_Type_commit(&individualtype);
+
+    PGADebugExited("PGABinaryBuildDatatype");
+
+    return (individualtype);
 }
 
 
