@@ -106,9 +106,9 @@ typedef struct
         int     PGAFuncNum;
 } PGAFuncRec;
 
-int        PGANumFcns;
+int        PGANumFcns = 0;
 
-PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
+PGAFuncRec PGAFuncIndex[] =
 {
 /* Binary Routines 100 - 149 */
         { "PGABinaryCreateString",          100 },
@@ -192,6 +192,9 @@ PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
         { "PGACreateIndividual",            303 },
         { "PGAGetRandomInitFlag",           304 },
         { "PGASetRandomInitFlag",           305 },
+        { "PGASetNumAuxEval",               306 },
+        { "PGASetNumConstraint",            307 },
+        { "PGASetSumConstraintsFlag",       308 },
 
         /* cross.c */
         { "PGACrossover",                   310 },
@@ -203,6 +206,8 @@ PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
         { "PGASetUniformCrossoverProb",     316 },
 
         /* pop.c */
+        { "PGA_NSGA_II_Replacement",            317 },
+        { "PGAPairwiseBestReplacement",         318 },
         { "PGARestrictedTournamentReplacement", 319 },
         { "PGASortPop",                         320 },
         { "PGAGetPopSize",                      321 },
@@ -215,7 +220,7 @@ PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
         { "PGASetRTRWindowSize",                328 },
         { "PGAGetRTRWindowSize",                329 },
 
-        /* mutation.c */
+        /* mutation.c */  /* 330-338, 400--499 */
         { "PGAMutate",                      330 },
         { "PGAGetMutationType",             331 },
         { "PGAGetMutationRealValue",        332 },
@@ -225,8 +230,10 @@ PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
         { "PGASetMutationRealValue",        336 },
         { "PGASetMutationIntegerValue",     337 },
         { "PGASetMutationProb",             338 },
-        { "PGASetMutationBoundedFlag",      400 },  /* 400--499 */
-        { "PGAGetMutationBoundedFlag",      401 },  /* 400--499 */
+        { "PGASetMutationBoundedFlag",      400 },
+        { "PGAGetMutationBoundedFlag",      401 },
+        { "PGASetMutationBounceBackFlag",   402 },
+        { "PGAGetMutationBounceBackFlag",   403 },
 
         /* duplcate.c */
         { "PGADuplicate",                   340 },
@@ -235,6 +242,7 @@ PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
         { "PGAGetNoDuplicatesFlag",         343 },
 
         /* pga.c */
+        { "PGARunMutationOnly",             349 },
         { "PGARunMutationAndCrossover",     350 },
         { "PGARunMutationOrCrossover",      351 },
         { "PGAUpdateGeneration",            352 },
@@ -294,13 +302,14 @@ PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
         { "PGAGetIntegerFromBinary",        508 },
         { "PGAGetIntegerFromGrayCode",      509 },
         { "PGAEvaluate",                    510 },
-	{ "PGAEvaluateSeq",                 515 },
-	{ "PGAEvaluateCoop",                516 },
-	{ "PGAEvaluateSlave",               517 },
         { "PGASetEvaluation",               511 },
         { "PGASetEvaluationUpToDateFlag",   512 },
         { "PGAGetEvaluation",               513 },
         { "PGAGetEvaluationUpToDateFlag",   514 },
+	{ "PGAEvaluateSeq",                 515 },
+	{ "PGAEvaluateCoop",                516 },
+	{ "PGAEvaluateSlave",               517 },
+        { "PGAGetAuxEvaluation",            518 },
 
         /* fitness.c */
         { "PGAFitness",                     520 },
@@ -361,6 +370,8 @@ PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
         { "PGAUpdateOnline",                719 },
         { "PGAUpdateOffline",               720 },
         { "PGAComputeSimilarity",           721 },
+        { "INDCopyIndividual",              722 },
+        { "PGAUpdateBest",                  723 },
 
         /* cmdline.c */
         { "PGAReadCmdLine",                 730 },
@@ -388,14 +399,6 @@ PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
         /* hamming.c */
         { "PGAHammingDistance",             800 },
 
-        /* heap.c */
-        { "PGADblHeapSort",                 810 },
-        { "PGADblHeapify",                  811 },
-        { "PGADblAdjustHeap",               812 },
-        { "PGAIntHeapSort",                 813 },
-        { "PGAIntHeapify",                  814 },
-        { "PGAIntAdjustHeap",               815 },
-
         /* report.c */
         { "PGAPrintPopulation",             820 },
         { "PGAPrintIndividual",             821 },
@@ -407,7 +410,10 @@ PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
         { "PGASetPrintOptions",             827 },
 
         /* user.c */
-        { "PGASetUserFunction",             830 }
+        { "PGASetUserFunction",             830 },
+
+        /* END GUARD */
+        { }
 };
 
 
@@ -423,9 +429,7 @@ PGAFuncRec PGAFuncIndex[PGA_DEBUG_MAXPGAPACKFUNCTIONS] =
 ****************************************************************************I*/
 void PGASortFuncNameIndex(PGAContext *ctx)
 {
-
-    /*  See how many functions are in the index */
-    for (PGANumFcns=0; PGAFuncIndex[PGANumFcns].PGAFuncName[0]; PGANumFcns++) ;
+    PGANumFcns = sizeof (PGAFuncIndex) / sizeof (PGAFuncRec);
 
     qsort(PGAFuncIndex, PGANumFcns, sizeof(PGAFuncRec),
 	  (int (*) (const void *, const void *)) &strcmp);
@@ -785,6 +789,7 @@ void PGASetDebugFlag11(PGAContext *ctx, int Flag)
    ctx->debug.PGADebugFlags[315] = Flag; /*PGASetCrossoverProb*/
    ctx->debug.PGADebugFlags[316] = Flag; /*PGASetUniformCrossoverProb*/
 
+   ctx->debug.PGADebugFlags[318] = Flag; /*PGAPairwiseBestReplacement*/
    ctx->debug.PGADebugFlags[319] = Flag; /*PGARestrictedTournamentReplacement*/
    ctx->debug.PGADebugFlags[320] = Flag; /*PGASortPop*/
    ctx->debug.PGADebugFlags[321] = Flag; /*PGAGetPopSize*/
@@ -805,14 +810,18 @@ void PGASetDebugFlag11(PGAContext *ctx, int Flag)
    ctx->debug.PGADebugFlags[336] = Flag; /*PGASetMutationRealValue*/
    ctx->debug.PGADebugFlags[337] = Flag; /*PGASetMutationIntegerValue*/
    ctx->debug.PGADebugFlags[338] = Flag; /*PGASetMutationProb*/
+
    ctx->debug.PGADebugFlags[400] = Flag; /*PGASetMutationBoundedFlag*/
    ctx->debug.PGADebugFlags[401] = Flag; /*PGAGetMutationBoundedFlag*/
+   ctx->debug.PGADebugFlags[402] = Flag; /*PGASetMutationBounceBackFlag*/
+   ctx->debug.PGADebugFlags[403] = Flag; /*PGAGetMutationBounceBackFlag*/
 
    ctx->debug.PGADebugFlags[340] = Flag; /*PGADuplicate*/
    ctx->debug.PGADebugFlags[341] = Flag; /*PGAChange*/
    ctx->debug.PGADebugFlags[342] = Flag; /*PGASetNoDuplicatesFlag*/
    ctx->debug.PGADebugFlags[343] = Flag; /*PGAGetNoDuplicatesFlag*/
 
+   ctx->debug.PGADebugFlags[349] = Flag; /*PGARunMutationOnly*/
    ctx->debug.PGADebugFlags[350] = Flag; /*PGARunMutationAndCrossover*/
    ctx->debug.PGADebugFlags[351] = Flag; /*PGARunMutationOrCrossover*/
    ctx->debug.PGADebugFlags[352] = Flag; /*PGAUpdateGeneration*/
@@ -887,9 +896,11 @@ void PGASetDebugFlag11(PGAContext *ctx, int Flag)
    ctx->debug.PGADebugFlags[707] = Flag; /*PGAGetMinMachineRealValue*/
    ctx->debug.PGADebugFlags[708] = Flag; /*PGADestroy*/
 
+   ctx->debug.PGADebugFlags[740] = Flag; /*PGADebugPrint*/
    ctx->debug.PGADebugFlags[741] = Flag; /*PGAGetDebugFlag*/
    ctx->debug.PGADebugFlags[742] = Flag; /*PGASetDebugFlag*/
    ctx->debug.PGADebugFlags[743] = Flag; /*PGAPrintDebugOptions*/
+   ctx->debug.PGADebugFlags[744] = Flag; /*PGASetDebugLevel*/
 
    ctx->debug.PGADebugFlags[800] = Flag; /*PGAHammingDistance*/
 
@@ -1033,6 +1044,8 @@ void PGASetDebugFlag32(PGAContext *ctx, int Flag)
    ctx->debug.PGADebugFlags[175] = Flag; /*PGAIntegerGeneDistance*/
    ctx->debug.PGADebugFlags[400] = Flag; /*PGASetMutationBoundedFlag*/
    ctx->debug.PGADebugFlags[401] = Flag; /*PGAGetMutationBoundedFlag*/
+   ctx->debug.PGADebugFlags[402] = Flag; /*PGASetMutationBounceBackFlag*/
+   ctx->debug.PGADebugFlags[403] = Flag; /*PGAGetMutationBounceBackFlag*/
 }
 
 /*I****************************************************************************
@@ -1423,12 +1436,6 @@ void PGASetDebugFlag60(PGAContext *ctx, int Flag)
 {
    ctx->debug.PGADebugFlags[320] = Flag; /*PGASortPop*/
    ctx->debug.PGADebugFlags[324] = Flag; /*PGAGetSortedPopIndex*/
-   ctx->debug.PGADebugFlags[810] = Flag; /*PGADblHeapSort*/
-   ctx->debug.PGADebugFlags[811] = Flag; /*PGADblHeapify*/
-   ctx->debug.PGADebugFlags[812] = Flag; /*PGADblAdjustHeap*/
-   ctx->debug.PGADebugFlags[813] = Flag; /*PGAIntHeapSort*/
-   ctx->debug.PGADebugFlags[814] = Flag; /*PGAIntHeapify*/
-   ctx->debug.PGADebugFlags[815] = Flag; /*PGAIntAdjustHeap*/
 }
 
 /*I****************************************************************************
