@@ -220,7 +220,7 @@ void PGASetIntegerInitPermute ( PGAContext *ctx, int min, int max)
       PGASetIntegerInitRange(ctx, low, high);
 
 ****************************************************************************U*/
-void PGASetIntegerInitRange (PGAContext *ctx, int *min, int *max)
+void PGASetIntegerInitRange (PGAContext *ctx, const int *min, const int *max)
 {
      int i;
 
@@ -423,61 +423,73 @@ void PGAIntegerCreateString (PGAContext *ctx, int p, int pop, int InitFlag)
 ****************************************************************************I*/
 int PGAIntegerMutation( PGAContext *ctx, int p, int pop, double mr )
 {
-     PGAInteger *c;
-     int i, j, temp;
-     int count = 0;
+    PGAInteger *c;
+    int i, j, temp;
+    int count = 0;
 
-     PGADebugEntered("PGAIntegerMutation");
+    PGADebugEntered("PGAIntegerMutation");
 
-     c = (PGAInteger *)PGAGetIndividual(ctx, p, pop)->chrom;
-     for(i=0; i<ctx->ga.StringLen; i++) {
+    c = (PGAInteger *)PGAGetIndividual(ctx, p, pop)->chrom;
+    for(i=0; i<ctx->ga.StringLen; i++) {
+        int old_value = c [i];
 
-         /* randomly choose an allele   */
-         if ( PGARandomFlip(ctx, mr) ) {
+        /* randomly choose an allele   */
+        if ( PGARandomFlip(ctx, mr) ) {
+            /* apply appropriate mutation operator */
+            switch (ctx->ga.MutationType) {
+            case PGA_MUTATION_CONSTANT:
+                /* add or subtract from allele */
+                if ( PGARandomFlip(ctx, .5) )
+                    c[i] += ctx->ga.MutateIntegerValue;
+                else
+                    c[i] -= ctx->ga.MutateIntegerValue;
+                break;
+            case PGA_MUTATION_PERMUTE:
+                /* could check for j == i if we were noble */
+	        /* edd: 16 Jun 2007  applying patch from Debian bug
+                 * report #333381 correcting an 'off-by-one' here
+		 * bu reducing StringLen by 1
+                 */
+                j = PGARandomInterval(ctx, 0, ctx->ga.StringLen - 1);
+                temp = c[i];
+                c[i] = c[j];
+                c[j] = temp;
+                break;
+            case PGA_MUTATION_RANGE:
+                c[i] = PGARandomInterval(ctx, ctx->init.IntegerMin[i],
+                                              ctx->init.IntegerMax[i]);
+                break;
+            default:
+                PGAError(ctx, "PGAIntegerMutation: Invalid value of "
+                         "ga.MutationType:", PGA_FATAL, PGA_INT,
+                         (void *) &(ctx->ga.MutationType));
+                break;
+            }
 
-             /* apply appropriate mutation operator */
-             switch (ctx->ga.MutationType) {
-             case PGA_MUTATION_CONSTANT:
-                 /* add or subtract from allele */             
-                 if ( PGARandomFlip(ctx, .5) )
-                      c[i] += ctx->ga.MutateIntegerValue;
-                 else
-                      c[i] -= ctx->ga.MutateIntegerValue;
-                 break;
-             case PGA_MUTATION_PERMUTE:
-                 /* could check for j == i if we were noble */
-	         /* edd: 16 Jun 2007  applying patch from Debian bug
-                    report #333381 correcting an 'off-by-one' here
-		    bu reducing StringLen by 1 */
-                 j = PGARandomInterval(ctx, 0, ctx->ga.StringLen - 1);
-                 temp = c[i];                 
-                 c[i] = c[j];
-                 c[j] = temp;                 
-                 break;
-             case PGA_MUTATION_RANGE:
-                 c[i] = PGARandomInterval(ctx, ctx->init.IntegerMin[i],
-                                               ctx->init.IntegerMax[i]);
-                 break;
-             default:
-                  PGAError(ctx, "PGAIntegerMutation: Invalid value of "
-                           "ga.MutationType:", PGA_FATAL, PGA_INT,
-                           (void *) &(ctx->ga.MutationType));
-                  break;
-             }
-
-             /* reset to min/max if bounded flag true and outside range */
-             if( ctx->ga.MutateBoundedFlag == PGA_TRUE ) {
-                 if( c[i] < ctx->init.IntegerMin[i])
-                     c[i] = ctx->init.IntegerMin[i];
-                 if( c[i] > ctx->init.IntegerMax[i])
-                     c[i] = ctx->init.IntegerMax[i];
-             }
-
-             count++;
-         }
-     }
-     PGADebugExited("PGAIntegerMutation");
-     return(count);
+            /* reset to min/max or bounce if outside range */
+            if (ctx->ga.MutateBoundedFlag || ctx->ga.MutateBounceFlag) {
+                if (c [i] < ctx->init.IntegerMin [i]) {
+                    if (ctx->ga.MutateBounceFlag) {
+                        c [i] = PGARandomInterval
+                            (ctx, ctx->init.IntegerMin [i], old_value);
+                    } else {
+                        c [i] = ctx->init.IntegerMin [i];
+                    }
+                }
+                if (c [i] > ctx->init.IntegerMax [i]) {
+                    if (ctx->ga.MutateBounceFlag) {
+                        c [i] = PGARandomInterval
+                            (ctx, old_value, ctx->init.IntegerMax [i]);
+                    } else {
+                        c [i] = ctx->init.IntegerMax [i];
+                    }
+                }
+            }
+            count++;
+        }
+    }
+    PGADebugExited("PGAIntegerMutation");
+    return(count);
 }
 
 /*I****************************************************************************
@@ -766,19 +778,21 @@ void PGAIntegerCopyString (PGAContext *ctx, int p1, int pop1, int p2, int pop2)
 ****************************************************************************I*/
 int PGAIntegerDuplicate( PGAContext *ctx, int p1, int pop1, int p2, int pop2)
 {
-     PGAInteger *a = (PGAInteger *)PGAGetIndividual(ctx, p1, pop1)->chrom;
-     PGAInteger *b = (PGAInteger *)PGAGetIndividual(ctx, p2, pop2)->chrom;
-     int i;
+    PGAInteger *a = (PGAInteger *)PGAGetIndividual (ctx, p1, pop1)->chrom;
+    PGAInteger *b = (PGAInteger *)PGAGetIndividual (ctx, p2, pop2)->chrom;
+    int count = 0;
 
-    PGADebugEntered("PGAIntegerDuplicate");
+    PGADebugEntered ("PGAIntegerDuplicate");
 
-     i = ctx->ga.StringLen-1;
-     if (a[0] == b[0])
-       for(; (i>0) && (a[i] == b[i]); i--);
+    for (count=0; count<ctx->ga.StringLen; count++) {
+        if (a [count] != b [count]) {
+            break;
+        }
+    }
 
     PGADebugExited("PGAIntegerDuplicate");
 
-     return((i==0) ? PGA_TRUE : PGA_FALSE);
+    return count == ctx->ga.StringLen ? PGA_TRUE : PGA_FALSE;
 }
 
 /*I****************************************************************************
@@ -848,41 +862,56 @@ void PGAIntegerInitString(PGAContext *ctx, int p, int pop)
 ****************************************************************************I*/
 MPI_Datatype PGAIntegerBuildDatatype(PGAContext *ctx, int p, int pop)
 {
-
-     int            counts[4];      /* Number of elements in each
-                                       block (array of integer) */
-     MPI_Aint       displs[4];      /* byte displacement of each
-                                       block (array of integer) */
-     MPI_Datatype   types[4];       /* type of elements in each block (array
-                                       of handles to datatype objects) */
-     MPI_Datatype   individualtype; /* new datatype (handle) */
-     PGAIndividual *traveller;      /* address of individual in question */
+    int            n = 6;
+    int            counts[7];      /* Number of elements in each
+                                      block (array of integer) */
+    MPI_Aint       displs[7];      /* byte displacement of each
+                                      block (array of integer) */
+    MPI_Datatype   types[7];       /* type of elements in each block (array
+                                      of handles to datatype objects) */
+    MPI_Datatype   individualtype; /* new datatype (handle) */
+    PGAIndividual *traveller;      /* address of individual in question */
 
     PGADebugEntered("PGAIntegerBuildDatatype");
 
-     traveller = PGAGetIndividual(ctx, p, pop);
-     MPI_Get_address(&traveller->evalfunc, &displs[0]);
-     counts[0] = 1;
-     types[0]  = MPI_DOUBLE;
+    traveller = PGAGetIndividual(ctx, p, pop);
+    MPI_Get_address(&traveller->evalue, &displs[0]);
+    counts[0] = 1;
+    types[0]  = MPI_DOUBLE;
 
-     MPI_Get_address(&traveller->fitness, &displs[1]);
-     counts[1] = 1;
-     types[1]  = MPI_DOUBLE;
+    MPI_Get_address(&traveller->fitness, &displs[1]);
+    counts[1] = 1;
+    types[1]  = MPI_DOUBLE;
 
-     MPI_Get_address(&traveller->evaluptodate, &displs[2]);
-     counts[2] = 1;
-     types[2]  = MPI_INT;
+    MPI_Get_address(&traveller->evaluptodate, &displs[2]);
+    counts[2] = 1;
+    types[2]  = MPI_INT;
 
-     MPI_Get_address(traveller->chrom, &displs[3]);
-     counts[3] = ctx->ga.StringLen;
-     types[3]  = MPI_LONG;
+    MPI_Get_address(traveller->chrom, &displs[3]);
+    counts[3] = ctx->ga.StringLen;
+    types[3]  = MPI_LONG;
 
-     MPI_Type_create_struct(4, counts, displs, types, &individualtype);
-     MPI_Type_commit(&individualtype);
+    MPI_Get_address(&traveller->auxtotal, &displs[4]);
+    counts[4] = 1;
+    types[4]  = MPI_DOUBLE;
+
+    MPI_Get_address(&traveller->auxtotalok, &displs[5]);
+    counts[5] = 1;
+    types[5]  = MPI_INT;
+
+    if (ctx->ga.NumAuxEval) {
+        MPI_Get_address(traveller->auxeval, &displs[6]);
+        counts[6] = ctx->ga.NumAuxEval;
+        types[6]  = MPI_DOUBLE;
+        n += 1;
+    }
+
+    MPI_Type_create_struct(n, counts, displs, types, &individualtype);
+    MPI_Type_commit(&individualtype);
 
     PGADebugExited("PGAIntegerBuildDatatype");
 
-     return (individualtype);
+    return (individualtype);
 }
 
 /*I****************************************************************************
@@ -918,3 +947,37 @@ double PGAIntegerGeneDistance (PGAContext *ctx, int p1, int pop1, int p2, int po
     return ret;
 }
 
+/*I****************************************************************************
+   PGAIntegerEuclidianDistance - Compute genetic difference of two strings.
+   This uses the Euclidian distance metric, the square-root of the sum
+   of all squared differences of each allele.
+
+   Inputs:
+      ctx   - context variable
+      p1    - first string index
+      pop1  - symbolic constant of the population the first string is in
+      p2    - second string index
+      pop2  - symbolic constant of the population the second string is in
+
+   Outputs:
+      genetic euclidian distance of the two strings
+
+   Example:
+      Use in PGASetUserFunction.
+
+****************************************************************************I*/
+double PGAIntegerEuclidianDistance (PGAContext *ctx, int p1, int pop1, int p2, int pop2)
+{
+    PGAInteger *c1 = (PGAInteger *)PGAGetIndividual (ctx, p1, pop1)->chrom;
+    PGAInteger *c2 = (PGAInteger *)PGAGetIndividual (ctx, p2, pop2)->chrom;
+    double ret = 0.0;
+    int i;
+
+    PGADebugEntered("PGAIntegerGeneDistance");
+    for (i=0; i<ctx->ga.StringLen; i++) {
+        ret += pow (c1 [i] - c2 [i], 2);
+    }
+    PGADebugExited("PGAIntegerGeneDistance");
+    return sqrt (ret);
+}
+
