@@ -11,10 +11,10 @@ Permission is hereby granted to use, reproduce, prepare derivative works, and
 to redistribute to others. This software was authored by:
 
 D. Levine
-Mathematics and Computer Science Division 
+Mathematics and Computer Science Division
 Argonne National Laboratory Group
 
-with programming assistance of participants in Argonne National 
+with programming assistance of participants in Argonne National
 Laboratory's SERS program.
 
 GOVERNMENT LICENSE
@@ -66,15 +66,15 @@ privately owned rights.
 
   Example:
     PGAContext *ctx;
-    double f(PGAContext *ctx, int p, int pop);
+    double f(PGAContext *ctx, int p, int pop, double *aux);
     :
     PGARunGM(ctx, f, MPI_COMM_WORLD);
 
 ****************************************************************************U*/
-void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int),
+void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int, double *),
 	      MPI_Comm comm)
 {
-    int       rank, Restarted, best_p;
+    int       rank, Restarted;
     void    (*CreateNewGeneration)(PGAContext *, int, int);
 
     /*  Let this be warned:
@@ -85,14 +85,32 @@ void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int),
 
     rank = PGAGetRank(ctx, comm);
 
-    PGAEvaluate(ctx, PGA_OLDPOP, f, comm);
-    if (rank == 0)
-	PGAFitness(ctx, PGA_OLDPOP);
+    if (rank == 0) {
+        if (ctx->fops.PreEval) {
+            int pop = PGA_OLDPOP;
+            (*ctx->fops.PreEval)(&ctx, &pop);
+        }
+        if (ctx->cops.PreEval) {
+            (*ctx->cops.PreEval)(ctx, PGA_OLDPOP);
+        }
+    }
 
-    if (PGAGetMutationOrCrossoverFlag(ctx)) 
+    PGAEvaluate (ctx, PGA_OLDPOP, f, comm);
+    if (rank == 0) {
+        int st = PGAGetSelectType (ctx);
+        PGAUpdateBest (ctx, PGA_OLDPOP);
+        if (st == PGA_SELECT_SUS || st == PGA_SELECT_PROPORTIONAL) {
+            PGAFitness (ctx, PGA_OLDPOP);
+        }
+    }
+
+    if (PGAGetMutationOnlyFlag(ctx)) {
+	CreateNewGeneration = PGARunMutationOnly;
+    } else if (PGAGetMutationOrCrossoverFlag(ctx)) {
 	CreateNewGeneration = PGARunMutationOrCrossover;
-    else
+    } else {
 	CreateNewGeneration = PGARunMutationAndCrossover;
+    }
 
     while (!PGADone(ctx, comm)) {
 	if (rank == 0) {
@@ -105,16 +123,27 @@ void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int),
 	    } else {
 		PGASelect(ctx, PGA_OLDPOP);
 		CreateNewGeneration(ctx, PGA_OLDPOP, PGA_NEWPOP);
+                if (ctx->fops.PreEval) {
+                    int pop = PGA_NEWPOP;
+                    (*ctx->fops.PreEval)(&ctx, &pop);
+                }
+                if (ctx->cops.PreEval) {
+                    (*ctx->cops.PreEval)(ctx, PGA_NEWPOP);
+                }
 	    }
 	}
 	MPI_Bcast(&Restarted, 1, MPI_INT, 0, comm);
 
 	PGAEvaluate(ctx, PGA_NEWPOP, f, comm);
-	if (rank == 0)
-	    PGAFitness(ctx, PGA_NEWPOP);
+	if (rank == 0) {
+            int st = PGAGetSelectType (ctx);
+            if (st == PGA_SELECT_SUS || st == PGA_SELECT_PROPORTIONAL) {
+                PGAFitness (ctx, PGA_NEWPOP);
+            }
+        }
 
 	/*  If the GA wasn't restarted, update the generation and print
-         *  stuff.  We do this because a restart is NOT counted as a 
+         *  stuff.  We do this because a restart is NOT counted as a
          *  complete generation.
 	 */
 	if (!Restarted) {
@@ -125,14 +154,42 @@ void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int),
     }
 
     if (rank == 0) {
-	best_p = PGAGetBestIndex(ctx, PGA_OLDPOP);
-	printf("The Best Evaluation: %e.\n",
-	       PGAGetEvaluation(ctx, best_p, PGA_OLDPOP));
-	printf("The Best String:\n");
-	PGAPrintString(ctx, stdout, best_p, PGA_OLDPOP);
-	fflush(stdout);
+        int pop = PGA_OLDPOP;
+        int numaux = PGAGetNumAuxEval (ctx);
+        int numcon = PGAGetNumConstraint (ctx);
+        if (numaux == numcon) {
+            int best_p = PGAGetBestIndex (ctx, pop);
+            printf
+                ( "The Best Evaluation: %e"
+                , PGAGetEvaluation (ctx, best_p, pop)
+                );
+            if (numaux) {
+                printf (" Constraints: %e", PGAGetAuxTotal (ctx, best_p, pop));
+            }
+            printf (".\n");
+            printf ("The Best String:\n");
+            PGAPrintString (ctx, stdout, best_p, pop);
+        } else {
+            int i, k;
+            PGAIndividual *ind = PGAGetIndividual (ctx, 0, pop);
+            for (k=0; k<numaux+1; k++) {
+                printf ("The Best (%d) evaluation: %e\n", k, ctx->rep.Best [k]);
+            }
+            printf ("The Nondominated Strings:\n");
+            for (i=0; i<ctx->ga.PopSize; i++) {
+                if (ind->rank == 0) {
+                    for (k=0; k<numaux+1; k++) {
+                        double e = (k==0) ? ind->evalue : ind->auxeval [k-1];
+                        printf ("F %5d %20.14e\n", k, e);
+                    }
+                    PGAPrintString (ctx, stdout, i, pop);
+                }
+                ind++;
+            }
+        }
+	fflush (stdout);
     }
-    PGADebugExited("PGARunGM");
+    PGADebugExited ("PGARunGM");
 }
 
 
@@ -153,29 +210,153 @@ void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int),
 
 ****************************************************************************I*/
 void PGAEvaluateSeq(PGAContext *ctx, int pop,
-		    double (*f)(PGAContext *, int, int))
+		    double (*f)(PGAContext *, int, int, double *))
 {
     int     p;
     double  e;
 
-    PGADebugEntered("PGAEvaluateSeq");
+    PGADebugEntered ("PGAEvaluateSeq");
 
     /*  Standard sequential evaluation.  */
-    if (ctx->sys.UserFortran == PGA_TRUE) {
-	for (p=1; p<=ctx->ga.PopSize; p++)
-	    if (!PGAGetEvaluationUpToDateFlag(ctx, p-1, pop)) {
-		e = (*((double(*)(void *, void *, void *))f))(&ctx, &p, &pop);
-		PGASetEvaluation (ctx, p-1, pop, e);
-	    }
-    } else {
-	for (p=0; p<ctx->ga.PopSize; p++) 
-	    if (!PGAGetEvaluationUpToDateFlag(ctx, p, pop)) {
-		e = (*f)(ctx, p, pop);
-		PGASetEvaluation(ctx, p, pop, e);
-	    }
+    for (p=0; p<ctx->ga.PopSize; p++) {
+        if (!PGAGetEvaluationUpToDateFlag(ctx, p, pop)) {
+            double *aux = PGAGetAuxEvaluation (ctx, p, pop);
+            if (ctx->sys.UserFortran == PGA_TRUE) {
+                int fp = p + 1;
+		e = (*((double(*)(void *, void *, void *, void *))f))
+                    (&ctx, &fp, &pop, aux);
+            } else {
+		e = (*f)(ctx, p, pop, aux);
+            }
+            PGASetEvaluation(ctx, p, pop, e, aux);
+            ctx->rep.nevals++;
+        }
     }
     PGADebugExited("PGAEvaluateSeq");
 }
+/*I****************************************************************************
+  PGABuildEvaluation - Build an MPI datatype for eval and auxeval.
+
+  Inputs:
+     ctx   - context variable
+     p     - index of string
+     pop   - symbolic constant of population string p is in
+
+  Outputs:
+     An MPI_Datatype.
+
+  Example:
+     PGAContext   *ctx;
+     int           p;
+     MPI_Datatype  dt;
+     :
+     dt = PGABuildEvaluation (ctx, p, pop);
+
+****************************************************************************I*/
+static
+MPI_Datatype PGABuildEvaluation (PGAContext *ctx, int p, int pop)
+{
+    int            n = 1;
+    int            counts[2];      /* Number of elements in each
+                                      block (array of integer) */
+    MPI_Aint       displs[2];      /* byte displacement of each
+                                      block (array of integer) */
+    MPI_Datatype   types[2];       /* type of elements in each block (array
+                                      of handles to datatype objects) */
+    MPI_Datatype   individualtype; /* new datatype (handle) */
+    PGAIndividual *traveller;      /* address of individual in question */
+
+    traveller = PGAGetIndividual(ctx, p, pop);
+    MPI_Get_address(&traveller->evalue, &displs[0]);
+    counts[0] = 1;
+    types[0]  = MPI_DOUBLE;
+
+    if (ctx->ga.NumAuxEval) {
+        MPI_Get_address (traveller->auxeval, &displs[1]);
+        counts[1] = ctx->ga.NumAuxEval;
+        types[1]  = MPI_DOUBLE;
+        n += 1;
+    }
+
+    MPI_Type_create_struct (n, counts, displs, types, &individualtype);
+    MPI_Type_commit (&individualtype);
+
+    return (individualtype);
+}
+
+/*U****************************************************************************
+  PGASendEvaluation - transmit evaluation and aux eval to another process
+
+  Category: Parallel
+
+  Inputs:
+    ctx  - context variable
+    p    - index of an individual
+    pop  - symbolic constant of the population
+    dest - ID of the process where this is going
+    tag  - MPI tag to send with the individual
+    comm - MPI communicator
+
+  Outputs:
+
+  Example:
+    PGAContext *ctx;
+    int p, dest;
+    :
+    dest = SelectAFreeProcessor();
+    PGASendEvaluation (ctx, p, PGA_NEWPOP, dest, PGA_COMM_EVALOFSTRING, comm);
+
+****************************************************************************U*/
+static
+void PGASendEvaluation (PGAContext *ctx, int p, int pop, int dest, int tag,
+                        MPI_Comm comm)
+{
+    MPI_Datatype individualtype;
+
+    individualtype = PGABuildEvaluation (ctx, p, pop);
+    MPI_Send (MPI_BOTTOM, 1, individualtype, dest, tag, comm);
+    MPI_Type_free (&individualtype);
+}
+
+/*U****************************************************************************
+  PGAReceiveEvaluation - receive evaluation and aux eval from another process
+
+  Category: Parallel
+
+  Inputs:
+    ctx    - contex variable
+    p      - index of an individual
+    pop    - symbolic constant of the population
+    source - ID of the process from which to receive
+    tag    - MPI tag to look for
+    status - pointer to an MPI status structure
+
+  Outputs:
+    string p in population pop is changed by side-effect.
+
+  Example:
+    Receive evaluation from sub-process and place it into the first temporary
+    location in PGA_NEWPOP.
+
+    PGAContext *ctx;
+    MPI_Comm    comm;
+    MPI_Status  status;
+    :
+    PGAReceiveEvaluation (ctx, PGA_TEMP1, PGA_NEWPOP, 0, PGA_COMM_EVALOFSTRING,
+                          comm, &status);
+
+****************************************************************************U*/
+static
+void PGAReceiveEvaluation (PGAContext *ctx, int p, int pop, int source, int tag,
+                           MPI_Comm comm, MPI_Status *status)
+{
+    MPI_Datatype individualtype;
+
+    individualtype = PGABuildEvaluation (ctx, p, pop);
+    MPI_Recv (MPI_BOTTOM, 1, individualtype, source, tag, comm, status);
+    MPI_Type_free (&individualtype);
+}
+
 
 
 /*I****************************************************************************
@@ -198,62 +379,70 @@ void PGAEvaluateSeq(PGAContext *ctx, int pop,
 
 ****************************************************************************I*/
 void PGAEvaluateCoop(PGAContext *ctx, int pop,
-		     double (*f)(PGAContext *, int, int), MPI_Comm comm)
+		     double (*f)(PGAContext *, int, int, double *),
+                     MPI_Comm comm)
 {
     MPI_Status      stat;
     int             p, fp, q;
     double          e;
     PGAIndividual  *ind;
 
-    PGADebugEntered("PGAEvaluateCoop");
+    PGADebugEntered ("PGAEvaluateCoop");
 
     q = -1;
 
     ind = PGAGetIndividual(ctx, 0, pop);
-    
+
     for (p=0; p<ctx->ga.PopSize;) {
 	while ((p<ctx->ga.PopSize) && (ind+p)->evaluptodate)  p++;
 	if (p<ctx->ga.PopSize) {
-	    PGASendIndividual(ctx, p, pop, 1, PGA_COMM_STRINGTOEVAL, comm);
+	    PGASendIndividual (ctx, p, pop, 1, PGA_COMM_STRINGTOEVAL, comm);
 	    q = p;
 	}
 	p++;
 	
 	while ((p<ctx->ga.PopSize) && (ind+p)->evaluptodate)  p++;
 	if (p<ctx->ga.PopSize) {
+            double *aux = PGAGetAuxEvaluation (ctx, p, pop);
 	    if (ctx->sys.UserFortran == PGA_TRUE) {
 		fp = p+1;
-		e = (*((double(*)(void *, void *, void *))f))(&ctx, &fp, &pop);
+		e = (*((double(*)(void *, void *, void *, void *))f))
+                    (&ctx, &fp, &pop, aux);
 	    } else {
-		e = (*f)(ctx, p, pop);
+		e = (*f)(ctx, p, pop, aux);
 	    }
-	    PGASetEvaluation(ctx, p, pop, e);
+	    PGASetEvaluation (ctx, p, pop, e, aux);
+            ctx->rep.nevals++;
 #if DEBUG_EVAL
-	    printf("%4d: %10.8e Local\n", p, e); fflush(stdout);
+	    printf ("%4d: %10.8e Local\n", p, e);
+            fflush (stdout);
 #endif
 	}
 	
 	if (q >= 0) {
-	    MPI_Recv(&e, 1, MPI_DOUBLE, 1, PGA_COMM_EVALOFSTRING, comm, &stat);
-	    PGASetEvaluation(ctx, q, pop, e);
+            PGAReceiveEvaluation
+                (ctx, q, pop, 1, PGA_COMM_EVALOFSTRING, comm, &stat);
+            PGASetEvaluationUpToDateFlag (ctx, q, pop, PGA_TRUE);
+            ctx->rep.nevals++;
 #if DEBUG_EVAL
-	    printf("%4d: %10.8e Slave %d\n", p, e, 1); fflush(stdout);
+	    printf ("%4d: %10.8e Slave %d\n", p, e, 1);
+            fflush (stdout);
 #endif
 	    q = -1;
 	}
-    }   
-    
-    /*  Release the slave  */
-    MPI_Send(&q, 1, MPI_INT, 1, PGA_COMM_DONEWITHEVALS, comm);
+    }
 
-    PGADebugExited("PGAEvaluateCoop");
+    /*  Release the slave  */
+    MPI_Send (&q, 1, MPI_INT, 1, PGA_COMM_DONEWITHEVALS, comm);
+
+    PGADebugExited ("PGAEvaluateCoop");
 }
 
 
 
 /*I****************************************************************************
    PGAEvaluateMS - Internal evaluation function.  Evaluates all strings
-   that need evaluating using three or more processors.  Operates in a 
+   that need evaluating using three or more processors.  Operates in a
    standard master-slave execution method.
 
    Category: Fitness & Evaluation
@@ -270,13 +459,14 @@ void PGAEvaluateCoop(PGAContext *ctx, int pop,
 
 ****************************************************************************I*/
 void PGAEvaluateMS(PGAContext *ctx, int pop,
-		   double (*f)(PGAContext *c, int p, int pop), MPI_Comm comm)
+		   double (*f)(PGAContext *c, int p, int pop, double *),
+                   MPI_Comm comm)
 {
     int    *work;
     int     i, k, s, p, size, sentout;
-    double  e;
     MPI_Status stat;
     PGAIndividual *ind;
+    PGAIndividual *tmp1 = PGAGetIndividual(ctx, PGA_TEMP1, pop);
 
     PGADebugEntered("PGAEvaluateMS");
 
@@ -287,11 +477,11 @@ void PGAEvaluateMS(PGAContext *ctx, int pop,
 	PGAError(ctx, "PGAEvaluateMS:  Couldn't allocate work array",
 		 PGA_FATAL, PGA_VOID, NULL);
     }
-    
+
     sentout = 0;
     s = 1;
     ind = PGAGetIndividual(ctx, 0, pop);
-    
+
     /*  Send strings to all processes, since they are all unused.  */
     for (k=0; ((k<ctx->ga.PopSize) && (s<size)); k++) {
 	if ((ind+k)->evaluptodate == PGA_FALSE) {
@@ -304,56 +494,60 @@ void PGAEvaluateMS(PGAContext *ctx, int pop,
 	    s++;
 	}
     }
-    
+
     /*  Move to the next string to be evaluated.  Notice that all we need
-     *  to do is skip any strings that are already evaluated, unlike 
+     *  to do is skip any strings that are already evaluated, unlike
      *  below, where we need to _first_ go to the next string, then
      *  skip any that are up to date.
      */
     while ((k<ctx->ga.PopSize) && (ind+k)->evaluptodate)  k++;
-    
+
     /*  While there are still unevaluated individuals, receive whatever
-     *  is waiting, then immediately send a new string to it.  This 
+     *  is waiting, then immediately send a new string to it.  This
      *  implicitly will balance the load across the machines, as we
      *  initially sent a string to _each_ process, so _each_ process
      *  will return an evaluation and get a new one immediately.
      */
     while(k<ctx->ga.PopSize) {
 	/*  Receive the next evaluated string.  */
-	MPI_Recv(&e, 1, MPI_DOUBLE, MPI_ANY_SOURCE, PGA_COMM_EVALOFSTRING,
-		 comm, &stat);
-	p = work[stat.MPI_SOURCE];
-	PGASetEvaluation(ctx, p, pop, e);
-	
+        PGAReceiveEvaluation
+            ( ctx, PGA_TEMP1, pop
+            , MPI_ANY_SOURCE, PGA_COMM_EVALOFSTRING, comm, &stat
+            );
+	p = work [stat.MPI_SOURCE];
+        PGASetEvaluation (ctx, p, pop, tmp1->evalue, tmp1->auxeval);
+        ctx->rep.nevals++;
 #if DEBUG_EVAL
 	printf("%4d: %10.8e Slave %d  Sent %d\n", work[stat.MPI_SOURCE],
 	       e, stat.MPI_SOURCE, k); fflush(stdout);
 #endif
-	
 	/*  Immediately send another string to be evaluated.  */
-	work[stat.MPI_SOURCE] = k;
-	PGASendIndividual(ctx, k, pop, stat.MPI_SOURCE,
-			  PGA_COMM_STRINGTOEVAL, comm);
+	work [stat.MPI_SOURCE] = k;
+	PGASendIndividual (ctx, k, pop, stat.MPI_SOURCE,
+			   PGA_COMM_STRINGTOEVAL, comm);
 	
 	/*  Find the next unevaluated individual  */
 	k++;
 	while ((k<ctx->ga.PopSize) && (ind+k)->evaluptodate)  k++;
     }
-    
+
     /*  All strings have been sent out.  Wait for them to be done.  */
     while(sentout > 0) {
-	MPI_Recv(&e, 1, MPI_DOUBLE, MPI_ANY_SOURCE, PGA_COMM_EVALOFSTRING,
-		 comm, &stat);
-	p = work[stat.MPI_SOURCE];
-	PGASetEvaluation(ctx, p, pop, e);
+        PGAReceiveEvaluation
+            ( ctx, PGA_TEMP1, pop
+            , MPI_ANY_SOURCE, PGA_COMM_EVALOFSTRING, comm, &stat
+            );
+        p = work [stat.MPI_SOURCE];
+        PGASetEvaluation (ctx, p, pop, tmp1->evalue, tmp1->auxeval);
+        ctx->rep.nevals++;
 	sentout--;
 #if DEBUG_EVAL
-	printf("%4d: %10.8e Slave %d\n", 
+	printf("%4d: %10.8e Slave %d\n",
 	       work[stat.MPI_SOURCE], e, stat.MPI_SOURCE); fflush(stdout);
 #endif
     }
     free(work);
-    
+
     /*  Release the slaves.  */
     for (i=1; i<size; i++)
 	MPI_Send(&i, 1, MPI_INT, i, PGA_COMM_DONEWITHEVALS, comm);
@@ -377,7 +571,8 @@ void PGAEvaluateMS(PGAContext *ctx, int pop,
 
 ****************************************************************************I*/
 void PGAEvaluateSlave(PGAContext *ctx, int pop,
-		      double (*f)(PGAContext *, int, int), MPI_Comm comm)
+		      double (*f)(PGAContext *, int, int, double *),
+                      MPI_Comm comm)
 {
     MPI_Status  stat;
     int         k;
@@ -388,15 +583,19 @@ void PGAEvaluateSlave(PGAContext *ctx, int pop,
     k = PGA_TEMP1;
     MPI_Probe(0, MPI_ANY_TAG, comm, &stat);
     while (stat.MPI_TAG == PGA_COMM_STRINGTOEVAL) {
+        double *aux;
 	PGAReceiveIndividual(ctx, PGA_TEMP1, pop, 0, PGA_COMM_STRINGTOEVAL,
 			     comm, &stat);
 
+        aux = PGAGetAuxEvaluation (ctx, PGA_TEMP1, pop);
 	if (ctx->sys.UserFortran == PGA_TRUE)
-	    e = (*((double(*)(void *, void *, void *))f))(&ctx, &k, &pop);
-	else 
-	    e = (*f)(ctx, PGA_TEMP1, pop);
+	    e = (*((double(*)(void *, void *, void *, void *))f))
+                (&ctx, &k, &pop, aux);
+	else
+	    e = (*f)(ctx, PGA_TEMP1, pop, aux);
+        PGASetEvaluation(ctx, PGA_TEMP1, pop, e, aux);
 
-	MPI_Send(&e, 1, MPI_DOUBLE, 0, PGA_COMM_EVALOFSTRING, comm);
+        PGASendEvaluation (ctx, PGA_TEMP1, pop, 0, PGA_COMM_EVALOFSTRING, comm);
 	MPI_Probe(0, MPI_ANY_TAG, comm, &stat);
     }
     MPI_Recv(&k, 1, MPI_INT, 0, PGA_COMM_DONEWITHEVALS, comm, &stat);
@@ -441,7 +640,7 @@ void PGAEvaluateSlave(PGAContext *ctx, int pop,
 
 ****************************************************************************U*/
 void PGAEvaluate(PGAContext *ctx, int pop,
-		 double (*f)(PGAContext *, int, int), MPI_Comm comm)
+		 double (*f)(PGAContext *, int, int, double *), MPI_Comm comm)
 {
     int  rank, size;
 
@@ -455,7 +654,7 @@ void PGAEvaluate(PGAContext *ctx, int pop,
 	    PGAEvaluateSeq(ctx, pop, f);
 	if (size == 2)
 	    PGAEvaluateCoop(ctx, pop, f, comm);
-	if (size > 2) 
+	if (size > 2)
 	    PGAEvaluateMS(ctx, pop, f, comm);
     } else {
 	PGAEvaluateSlave(ctx, pop, f, comm);
@@ -489,7 +688,7 @@ void PGAEvaluate(PGAContext *ctx, int pop,
 MPI_Datatype PGABuildDatatype(PGAContext *ctx, int p, int pop)
 {
     PGADebugEntered("PGABuildDatatype");
-    
+
     PGADebugExited("PGABuildDatatype");
 
     return((*ctx->cops.BuildDatatype)(ctx, p, pop));
@@ -529,7 +728,7 @@ void PGASendIndividual(PGAContext *ctx, int p, int pop, int dest, int tag,
     individualtype = PGABuildDatatype(ctx, p, pop);
     MPI_Send(MPI_BOTTOM, 1, individualtype, dest, tag, comm);
     MPI_Type_free(&individualtype);
-    
+
     PGADebugExited("PGASendIndividual");
 }
 
@@ -660,13 +859,14 @@ void PGASendReceiveIndividual(PGAContext *ctx, int send_p, int send_pop,
     PGARunIM(ctx, f, comm);
 
 ****************************************************************************I*/
-void PGARunIM(PGAContext *ctx, double (*f)(PGAContext *c, int p, int pop),
+void PGARunIM(PGAContext *ctx,
+              double (*f)(PGAContext *c, int p, int pop, double *aux),
               MPI_Comm tcomm)
 {
     /* Based on ctx->par.topology this routine will need to create the
        appropriate communicator out of tcomm
     */
-    
+
      PGADebugEntered("PGARunIM");
      PGAError (ctx, "PGARunIM: Island model not implemented",
                PGA_FATAL, PGA_VOID, NULL);
@@ -696,7 +896,8 @@ void PGARunIM(PGAContext *ctx, double (*f)(PGAContext *c, int p, int pop),
     PGARunNM(ctx, f, comm);
 
 ****************************************************************************I*/
-void PGARunNM(PGAContext *ctx, double (*f)(PGAContext *c, int p, int pop),
+void PGARunNM(PGAContext *ctx,
+              double (*f)(PGAContext *c, int p, int pop, double *aux),
               MPI_Comm tcomm)
 {
     /* Based on ctx->par.topology this routine will need to create the
@@ -737,16 +938,16 @@ void PGARunNM(PGAContext *ctx, double (*f)(PGAContext *c, int p, int pop),
 int PGAGetRank (PGAContext *ctx, MPI_Comm comm)
 {
     int rank;
-    
+
     PGADebugEntered("PGAGetRank");
 
     if (comm == MPI_COMM_NULL)
 	rank = 0;
     else
 	MPI_Comm_rank(comm, &rank);
-    
+
     PGADebugExited("PGAGetRank");
-    
+
     return(rank);
 }
 
@@ -794,7 +995,7 @@ int PGAGetNumProcs (PGAContext *ctx, MPI_Comm comm)
 /*I****************************************************************************
    PGASetNumIslands - Set the number of islands to use in an island model
    GA. The default is one.  Currently must be the same as the number of
-   processes in the default communicator.  
+   processes in the default communicator.
 
    Category: Parallel
 
@@ -967,7 +1168,7 @@ void PGASetCommunicator( PGAContext *ctx, MPI_Comm comm)
 /*U****************************************************************************
    PGAGetCommunicator - Returns the default communicator used when PGARun is
    called.
-   
+
    Category: Parallel
 
    Inputs:
