@@ -71,7 +71,8 @@ privately owned rights.
     PGADestroy(ctx);
 
 ****************************************************************************U*/
-void PGARun(PGAContext *ctx, double (*evaluate)(PGAContext *c, int p, int pop))
+void PGARun (PGAContext *ctx,
+             double (*evaluate)(PGAContext *c, int p, int pop, double *))
 {
      MPI_Comm comm;                  /* value of default communicator */
      int nprocs;                     /* number of processes in above  */
@@ -177,10 +178,16 @@ void PGARunMutationAndCrossover (PGAContext *ctx, int oldpop, int newpop)
 
     popsize = PGAGetPopSize (ctx);
     numreplace = PGAGetNumReplaceValue (ctx);
-    /*** first, copy n best strings (sorted by fitness) to new pop ***/
+    /*** first, copy n best strings to new pop ***/
     /*** Note that we do not need to do this for PGA_POPREPL_RTR   ***/
+    /*** And neither for PGA_POPREPL_PAIRWISE_BEST                 ***/
+    /*** And neither for PGA_POPREPL_NSGA_II                       ***/
     n = popsize - numreplace;
-    if (ctx->ga.PopReplace != PGA_POPREPL_RTR) {
+    if (  ctx->ga.PopReplace != PGA_POPREPL_RTR
+       && ctx->ga.PopReplace != PGA_POPREPL_PAIRWISE_BEST
+       && ctx->ga.PopReplace != PGA_POPREPL_NSGA_II
+       )
+    {
         PGASortPop (ctx, oldpop);
         for (i=0; i < n; i++) {
             j = PGAGetSortedPopIndex (ctx, i);
@@ -255,10 +262,16 @@ void PGARunMutationOrCrossover (PGAContext *ctx, int oldpop, int newpop)
 
     popsize = PGAGetPopSize (ctx);
     numreplace = PGAGetNumReplaceValue (ctx);
-    /*** first, copy n best strings (sorted by fitness) to new pop ***/
+    /*** first, copy n best strings to new pop ***/
     /*** Note that we do not need to do this for PGA_POPREPL_RTR   ***/
+    /*** And neither for PGA_POPREPL_PAIRWISE_BEST                 ***/
+    /*** And neither for PGA_POPREPL_NSGA_II                       ***/
     n = popsize - numreplace;
-    if (ctx->ga.PopReplace != PGA_POPREPL_RTR) {
+    if (  ctx->ga.PopReplace != PGA_POPREPL_RTR
+       && ctx->ga.PopReplace != PGA_POPREPL_PAIRWISE_BEST
+       && ctx->ga.PopReplace != PGA_POPREPL_NSGA_II
+       )
+    {
         PGASortPop (ctx, oldpop);
         for (i=0; i < n; i++) {
             j = PGAGetSortedPopIndex (ctx, i);
@@ -309,6 +322,65 @@ void PGARunMutationOrCrossover (PGAContext *ctx, int oldpop, int newpop)
 }
 
 
+/*U****************************************************************************
+  PGARunMutationOnly - Performs only mutation
+  Assumes PGASelect has been called.
+
+  Category: Generation
+
+  Inputs:
+    ctx - context variable
+    oldpop - symbolic constant of old population
+    newpop - symbolic constant of new population
+
+  Outputs:
+    newpop is modified by side-effect.
+
+  Example:
+    PGAContext *ctx,
+    :
+    PGARunMutationOnly(ctx, PGA_OLDPOP, PGA_NEWPOP);
+
+****************************************************************************U*/
+void PGARunMutationOnly (PGAContext *ctx, int oldpop, int newpop)
+{
+    int i, j, n, m;
+    int popsize, numreplace;
+
+    PGADebugEntered ("PGARunMutationOnly");
+
+    popsize = PGAGetPopSize (ctx);
+    numreplace = PGAGetNumReplaceValue (ctx);
+    /*** first, copy n best strings to new pop ***/
+    /*** Note that we do not need to do this for PGA_POPREPL_RTR   ***/
+    /*** And neither for PGA_POPREPL_PAIRWISE_BEST                 ***/
+    /*** And neither for PGA_POPREPL_NSGA_II                       ***/
+    n = popsize - numreplace;
+    if (  ctx->ga.PopReplace != PGA_POPREPL_RTR
+       && ctx->ga.PopReplace != PGA_POPREPL_PAIRWISE_BEST
+       && ctx->ga.PopReplace != PGA_POPREPL_NSGA_II
+       )
+    {
+        PGASortPop (ctx, oldpop);
+        for (i=0; i < n; i++) {
+            j = PGAGetSortedPopIndex (ctx, i);
+            PGACopyIndividual (ctx, j, oldpop, i, newpop);
+        }
+    }
+    /*** reproduce with mutation only to create rest of the new population ***/
+    while (n < popsize) {
+        m = PGASelectNextIndex (ctx, oldpop);
+        PGACopyIndividual (ctx, m, oldpop, n, newpop);
+        PGAMutate (ctx, n, newpop);
+        while (PGADuplicate (ctx, n, newpop, newpop, n))
+             PGAChange (ctx, n, newpop);
+        n++;
+    }
+
+    PGADebugExited ("PGARunMutationOnly");
+}
+
+
 /*U****************************************************************************
   PGAUpdateGeneration - updates internal data structures for the next
   genetic algorithm iteration, and checks if the termination conditions, both
@@ -330,56 +402,100 @@ void PGARunMutationOrCrossover (PGAContext *ctx, int oldpop, int newpop)
     PGAUpdateGeneration(ctx, MPI_COMM_WORLD);
 
 ****************************************************************************U*/
-void PGAUpdateGeneration(PGAContext *ctx, MPI_Comm comm)
+void PGAUpdateGeneration (PGAContext *ctx, MPI_Comm comm)
 {
     PGAIndividual *temp;
-    int i, rank;
+    int rank;
 
-    PGADebugEntered("PGAUpdateGeneration");
-    PGADebugPrint( ctx, PGA_DEBUG_PRINTVAR,"PGAUpdateGeneration",
-                  "ga.iter = ", PGA_INT, (void *) &(ctx->ga.iter) );
+    PGADebugEntered ("PGAUpdateGeneration");
+    PGADebugPrint (ctx, PGA_DEBUG_PRINTVAR,"PGAUpdateGeneration",
+                   "ga.iter = ", PGA_INT, (void *) &(ctx->ga.iter));
 
-    rank = PGAGetRank(ctx, comm);
+    rank = PGAGetRank (ctx, comm);
 
     ctx->ga.iter++;
 
     if (rank == 0) {
+        double oldbest [ctx->ga.NumAuxEval + 1];
+        /* The three replacement schemes PGA_POPREPL_PAIRWISE_BEST,
+         * PGA_POPREPL_RTR, and PGA_POPREPL_NSGA_II all replace some new
+         * individuals from PGA_NEWPOP into PGA_OLDPOP. Then
+         * PGA_NEWPOP/PGA_OLDPOP are switched (resulting in the current
+         * population to be PGA_NEWPOP). They are switched *again* at
+         * the end of this function to be ready for the next generation.
+         * Note that these functions may not use the fitness because
+         * this is not comparable across populations.
+         */
         if (ctx->ga.PopReplace == PGA_POPREPL_RTR) {
-            /* This replaces the selected individuals into OLDPOP and
-             * then switches OLDPOP/NEWPOP
+            /* This replaces the selected individuals into OLDPOP.
+             * Then OLDPOP/NEWPOP are exchanged
              */
-            PGARestrictedTournamentReplacement(ctx);
+            PGARestrictedTournamentReplacement (ctx);
+        }
+        else if (ctx->ga.PopReplace == PGA_POPREPL_PAIRWISE_BEST) {
+            /* This compares individual in OLDPOP with the individual
+             * with same index in NEWPOP and puts the better one into
+             * OLDPOP.
+             * Then OLDPOP/NEWPOP are exchanged
+             */
+            PGAPairwiseBestReplacement (ctx);
+        }
+        else if (ctx->ga.PopReplace == PGA_POPREPL_NSGA_II) {
+            /* This performs nondominated sorting and replaces the best
+             * individuals over both populations into OLDPOP.
+             * Then OLDPOP/NEWPOP are exchanged
+             */
+            PGA_NSGA_II_Replacement (ctx);
         }
 
-	if (ctx->rep.PrintOptions & PGA_REPORT_AVERAGE)
+	if (ctx->rep.PrintOptions & PGA_REPORT_AVERAGE) {
 	    PGAUpdateAverage(ctx, PGA_NEWPOP);
+        }
 
-	if (ctx->rep.PrintOptions & PGA_REPORT_ONLINE)
+	if (ctx->rep.PrintOptions & PGA_REPORT_ONLINE) {
 	    PGAUpdateOnline(ctx, PGA_NEWPOP);
+        }
 
-	if (ctx->rep.PrintOptions & PGA_REPORT_OFFLINE)
+	if (ctx->rep.PrintOptions & PGA_REPORT_OFFLINE) {
 	    PGAUpdateOffline(ctx, PGA_NEWPOP);
+        }
 
+
+        memcpy (oldbest, ctx->rep.Best, sizeof (oldbest));
+        PGAUpdateBest (ctx, PGA_NEWPOP);
 	if ((ctx->ga.StoppingRule & PGA_STOP_NOCHANGE) || ctx->ga.restart) {
-	    i = PGAGetBestIndex(ctx, PGA_NEWPOP);
-	    if (ctx->rep.Best == PGAGetEvaluation(ctx, i, PGA_NEWPOP))
-		ctx->ga.ItersOfSame++;
-	    else {
-		ctx->rep.Best = PGAGetEvaluation(ctx, i, PGA_NEWPOP);
-		ctx->ga.ItersOfSame = 1;
-	    }
+            double *best = ctx->rep.Best;
+            int k;
+            int equal = 1;
+            for (k=0; k<ctx->ga.NumAuxEval+1;k++) {
+                if (! (  (isnan (oldbest [k]) && isnan (best [k]))
+                      || oldbest [k] == best [k]
+                      )
+                   )
+                {
+                    equal = 0;
+                    break;
+                }
+            }
+            if (equal) {
+                ctx->ga.ItersOfSame++;
+            } else {
+                ctx->ga.ItersOfSame = 1;
+            }
 	}
 
 	if (ctx->ga.StoppingRule & PGA_STOP_TOOSIMILAR)
-	    ctx->ga.PercentSame = PGAComputeSimilarity(ctx, ctx->ga.newpop);
+	    ctx->ga.PercentSame = PGAComputeSimilarity (ctx, PGA_NEWPOP);
 
 	/*  Clear this twice in case the user EOG calls PGASelect.  */
 	ctx->ga.SelectIndex = 0;
 
-	if (ctx->fops.EndOfGen)
+	if (ctx->fops.EndOfGen) {
 	    (*ctx->fops.EndOfGen)(&ctx);
-	if (ctx->cops.EndOfGen)
+        }
+	if (ctx->cops.EndOfGen) {
 	    (*ctx->cops.EndOfGen)(ctx);
+        }
 
 	ctx->ga.SelectIndex = 0;
 	temp           = ctx->ga.oldpop;
@@ -387,7 +503,7 @@ void PGAUpdateGeneration(PGAContext *ctx, MPI_Comm comm)
 	ctx->ga.newpop = temp;
     }
 
-    PGADebugExited("PGAUpdateGeneration");
+    PGADebugExited ("PGAUpdateGeneration");
 }
 
 
@@ -566,6 +682,31 @@ int PGAGetGAIterValue (PGAContext *ctx)
     return(ctx->ga.iter);
 }
 
+/*U***************************************************************************
+  PGAGetEvalCount - returns the number of function evaluations
+
+   Category: Generation
+
+   Inputs:
+      ctx - context variable
+
+   Outputs:
+      The number of function evaluations
+
+   Example:
+      PGAContext *ctx;
+      int g;
+      :
+      g = PGAGetEvalCount(ctx);
+
+***************************************************************************U*/
+int PGAGetEvalCount (PGAContext *ctx)
+{
+    PGAFailIfNotSetUp("PGAGetEvalCount");
+
+    return(ctx->rep.nevals);
+}
+
 /*U****************************************************************************
   PGASetMutationOrCrossoverFlag - A boolean flag to indicate if recombination
   uses exactly one of crossover or mutation on selected strings.
@@ -723,3 +864,72 @@ int PGAGetMutationAndCrossoverFlag (PGAContext *ctx)
     return(!ctx->ga.MutateOnlyNoCross);
 }
 
+/*U****************************************************************************
+  PGASetMutationOnlyFlag - A boolean flag to indicate that recombination
+  uses mutation only.
+
+   Category: Generation
+
+   Inputs:
+      ctx  - context variable
+      flag - PGA_TRUE (default) or PGA_FALSE
+
+   Outputs:
+      None
+
+   Example:
+      Set the genetic algorithm to use mutation only.
+
+      PGAContext *ctx;
+      :
+      PGASetMutationOnlyFlag(ctx,PGA_TRUE);
+
+****************************************************************************U*/
+void PGASetMutationOnlyFlag (PGAContext *ctx, int flag)
+{
+     switch (flag)
+     {
+     case PGA_TRUE:
+     case PGA_FALSE:
+          ctx->ga.MutateOnly = flag;
+          break;
+     default:
+          PGAError (ctx, "PGASetMutationOrCrossoverFlag: Invalid value of "
+                    "flag:", PGA_FATAL, PGA_INT, (void *) &flag);
+          break;
+     }
+}
+
+/*U***************************************************************************
+   PGAGetMutationOnlyFlag - Returns true if only mutation is used
+
+   Category: Generation
+
+   Inputs:
+      ctx - context variable
+
+   Outputs:
+      Returns PGA_TRUE if only mutation is applied.
+      Otherwise, returns PGA_FALSE
+
+   Example:
+      PGAContext *ctx;
+      int mutateonly;
+      :
+      mutateonly = PGAGetMutationOnlyFlag(ctx);
+      switch (mutatetype) {
+      case PGA_TRUE:
+          printf("Mutating only\n");
+          break;
+      case PGA_FALSE:
+          printf("Mutating and/or crossover\n");
+          break;
+      }
+
+***************************************************************************U*/
+int PGAGetMutationOnlyFlag (PGAContext *ctx)
+{
+    PGAFailIfNotSetUp("PGAGetMutationOnlyFlag");
+    return(ctx->ga.MutateOnly);
+}
+
