@@ -11,10 +11,10 @@ Permission is hereby granted to use, reproduce, prepare derivative works, and
 to redistribute to others. This software was authored by:
 
 D. Levine
-Mathematics and Computer Science Division 
+Mathematics and Computer Science Division
 Argonne National Laboratory Group
 
-with programming assistance of participants in Argonne National 
+with programming assistance of participants in Argonne National
 Laboratory's SERS program.
 
 GOVERNMENT LICENSE
@@ -181,7 +181,6 @@ void PGASetRealInitPercent ( PGAContext *ctx, double *median, double *percent)
          offset = fabs(median[i] * percent[i]);
          ctx->init.RealMin[i] = median[i] - offset;
          ctx->init.RealMax[i] = median[i] + offset;
-         
     }
     ctx->init.RealType = PGA_RINIT_PERCENT;
 
@@ -224,7 +223,7 @@ void PGASetRealInitPercent ( PGAContext *ctx, double *median, double *percent)
      PGASetRealInitRange(ctx, low, high);
 
 ****************************************************************************U*/
-void PGASetRealInitRange (PGAContext *ctx, double *min, double *max)
+void PGASetRealInitRange (PGAContext *ctx, const double *min, const double *max)
 {
      int i;
     PGADebugEntered("PGASetRealInitRange");
@@ -386,9 +385,9 @@ void PGARealCreateString (PGAContext *ctx, int p, int pop, int initflag)
     PGAIndividual *new = PGAGetIndividual(ctx, p, pop);
     int i, fp;
     PGAReal *c;
-    
+
     PGADebugEntered("PGARealCreateString");
-    
+
     new->chrom = (void *) malloc (ctx->ga.StringLen * sizeof(PGAReal));
     if (new->chrom == NULL)
 	PGAError(ctx, "PGARealCreateString: No room to allocate new->chrom",
@@ -404,7 +403,7 @@ void PGARealCreateString (PGAContext *ctx, int p, int pop, int initflag)
     else
 	for (i=ctx->ga.StringLen-1; i>=0; i--)
 	    c[i] = 0.0;
-    
+
     PGADebugExited("PGARealCreateString");
 }
 
@@ -440,71 +439,264 @@ void PGARealCreateString (PGAContext *ctx, int p, int pop, int initflag)
       NumMutations = PGARealMutation( ctx, p, PGA_NEWPOP, .001 );
 
 ****************************************************************************I*/
-int PGARealMutation( PGAContext *ctx, int p, int pop, double mr )
+int PGARealMutation (PGAContext *ctx, int p, int pop, double mr)
 {
-     PGAReal *c;
-     int i;
-     int count = 0;
-     double val = 0.0;
+    PGAReal *c;
+    int i, j;
+    int count = 0;
+    double val = 0.0;
+    /* The following are used for DE variants */
+    int midx = 0;
+    int do_best = (ctx->ga.DEVariant == PGA_DE_VARIANT_BEST);
+    int nrand  = 2 * ctx->ga.DENumDiffs + (!do_best);
+    int maxidx = 2 * ctx->ga.DENumDiffs + 1;
+    PGAReal *indivs [maxidx];
+    int do_crossover = 1;
+    static double de_dither = 0.0;
+    static int last_iter = -1;
 
-     PGADebugEntered("PGARealMutation");
 
-     c = (PGAReal *)PGAGetIndividual(ctx, p, pop)->chrom;
-     for(i=0; i<ctx->ga.StringLen; i++) {
+    PGADebugEntered("PGARealMutation");
+    if (ctx->ga.MutationType == PGA_MUTATION_DE) {
+        int idx [maxidx];
+        PGASampleState sstate;
+        int best = PGAGetBestIndex (ctx, PGA_OLDPOP);
+        int avoid = 1;
 
-         /* randomly choose an allele   */
-         if ( PGARandomFlip(ctx, mr) ) {
+        if (ctx->ga.DEDither > 0) {
+            if (ctx->ga.DEDitherPerIndividual || last_iter != ctx->ga.iter) {
+                de_dither = ctx->ga.DEDither * (PGARandom01 (ctx, 0) - 0.5);
+                last_iter = ctx->ga.iter;
+            }
+        }
 
-             /* generate on range, or calculate multplier */
-             switch (ctx->ga.MutationType) {
-             case PGA_MUTATION_RANGE:
-                 c[i] = PGARandomUniform(ctx, ctx->init.RealMin[i],
-                                              ctx->init.RealMax[i]);
-                 break;
-             case PGA_MUTATION_CONSTANT:
-                 val = ctx->ga.MutateRealValue;
-                 break;
-             case PGA_MUTATION_UNIFORM:
-                 val = PGARandomUniform (ctx, 0.0, ctx->ga.MutateRealValue);
-                 break;
-             case PGA_MUTATION_GAUSSIAN:
-                 val = PGARandomGaussian(ctx, 0.0, ctx->ga.MutateRealValue);
-                 break;
-             default:
-                  PGAError(ctx, "PGARealMutation: Invalid value of "
-                           "ga.MutationType:", PGA_FATAL, PGA_INT,
-                           (void *) &(ctx->ga.MutationType));
-                  break;
-             }
+        /* We rely on the fact that we operate on an individual of
+         * PGA_NEWPOP and take data from PGA_OLDPOP. Assert this is the
+         * case.
+         */
+        if (pop != PGA_NEWPOP) {
+            PGAError
+                ( ctx, "PGARealMutation: Invalid value of pop:"
+                , PGA_FATAL, PGA_INT, (void *) &(pop)
+                );
+        }
 
-             /* apply multiplier calculated in switch above */
-             if ( (ctx->ga.MutationType == PGA_MUTATION_CONSTANT) ||
-                  (ctx->ga.MutationType == PGA_MUTATION_UNIFORM)  ||
-                  (ctx->ga.MutationType == PGA_MUTATION_GAUSSIAN)
-                ) {
-                 /* add/subtract from allele */
-                 if ( PGARandomFlip(ctx, .5) )
-                     c[i] += val*c[i];
-                 else
-                     c[i] -= val*c[i];
-             }
+        /* for BEST strategy we have to avoid collision with running
+         * index p *and* the best individual
+         */
+        if (do_best && best != p) {
+            avoid = 2;
+        }
+        /* Use (PopSize - avoid) to avoid collision with running index
+         * and with the best index (unless this is the same)
+         * We may use this form of selection of the indeces needed for
+         * DE for linear selection, for other selection schemes we use
+         * the selected individuals and re-sample if we get collisions.
+         */
+        if (ctx->ga.SelectType == PGA_SELECT_LINEAR) {
+            PGARandomSampleInit (ctx, &sstate, nrand, ctx->ga.PopSize - avoid);
+            for (i=0; i<nrand; i++) {
+                int rawidx = PGARandomNextSample (&sstate);
+                /* Avoid collision with p and optionally best, samples
+                 * are drawn with reduced upper bound, see
+                 * PGARandomSampleInit above
+                 */
+                if (rawidx >= p) {
+                    rawidx += 1;
+                }
+                /* The best individual can be in the part of the old
+                 * population that is copied verbatim to the new
+                 * population. In that case never increment the index
+                 */
+                if (do_best && best != p && rawidx >= best) {
+                    rawidx += 1;
+                }
+                idx [i] = rawidx;
+            }
+        } else {
+            for (i=0; i<nrand; i++) {
+                do {
+                    idx [i] = PGASelectNextIndex (ctx, PGA_OLDPOP);
+                } while
+                    (  idx [i] == p
+                    || (do_best && idx [i] == best)
+                    || (i > 0 && idx [i] == idx [i-1])
+                    || (i > 1 && idx [i] == idx [i-2])
+                    || (i > 2 && idx [i] == idx [i-3])
+                    || (i > 3 && idx [i] == idx [i-4])
+                    );
+            }
+        }
+        /* Since indices from PGARandomNextSample are
+         * returned in order we need to shuffle
+         */
+        for (i=0; i<nrand-1; i++) {
+            int tmp;
+            j = PGARandomInterval (ctx, i, nrand-1);
+            tmp = idx [j];
+            idx [j] = idx [i];
+            idx [i] = tmp;
+        }
+        /* Now we have a list of shuffled indexes that do not
+         * collide with one-another or with p or best
+         * Add best index as last to the list if do_best
+         */
+        if (do_best) {
+            idx [maxidx - 1] = best;
+        }
+        for (i=0; i<maxidx; i++) {
+            indivs [i] = (PGAReal *)
+                PGAGetIndividual (ctx, idx [i], PGA_OLDPOP)->chrom;
+        }
+        /* Index of allele that is mutated in any case */
+        midx = PGARandomInterval (ctx, 0, ctx->ga.StringLen - 1);
+    }
 
-             /* reset to min/max if bounded flag true and outside range */
-             if( ctx->ga.MutateBoundedFlag == PGA_TRUE ) {
-                if( c[i] < ctx->init.RealMin[i])
-                    c[i] = ctx->init.RealMin[i];
-                if( c[i] > ctx->init.RealMax[i])
-                    c[i] = ctx->init.RealMax[i];
-             }
+    c = (PGAReal *)PGAGetIndividual(ctx, p, pop)->chrom;
+    for(i=0; i<ctx->ga.StringLen; i++) {
+        double old_value = c [i];
+        int idx = i;
 
-             /* increment mutation count */
-             count++;
-         }
-     }
+        switch (ctx->ga.MutationType) {
+            case PGA_MUTATION_RANGE:
+            case PGA_MUTATION_CONSTANT:
+            case PGA_MUTATION_UNIFORM:
+            case PGA_MUTATION_GAUSSIAN:
+                /* randomly choose an allele   */
+                if ( PGARandomFlip(ctx, mr) ) {
+                    /* generate on range, or calculate multiplier */
+                    switch (ctx->ga.MutationType) {
+                    case PGA_MUTATION_RANGE:
+                        c [i] = PGARandomUniform(ctx, ctx->init.RealMin [i],
+                                                      ctx->init.RealMax [i]);
+                        break;
+                    case PGA_MUTATION_CONSTANT:
+                        val = ctx->ga.MutateRealValue;
+                        break;
+                    case PGA_MUTATION_UNIFORM:
+                        val = PGARandomUniform
+                            (ctx, 0.0, ctx->ga.MutateRealValue);
+                        break;
+                    case PGA_MUTATION_GAUSSIAN:
+                        val = PGARandomGaussian
+                            (ctx, 0.0, ctx->ga.MutateRealValue);
+                        break;
+                    }
+                    /* apply multiplier calculated in switch above */
+                    if ( (ctx->ga.MutationType == PGA_MUTATION_CONSTANT) ||
+                         (ctx->ga.MutationType == PGA_MUTATION_UNIFORM)  ||
+                         (ctx->ga.MutationType == PGA_MUTATION_GAUSSIAN)
+                       )
+                    {
+                         /* add/subtract from allele */
+                        if ( PGARandomFlip(ctx, .5) )
+                            c [i] += val * c [i];
+                        else
+                            c [i] -= val * c [i];
+                    }
+                    /* increment mutation count */
+                    count++;
+                }
+                break;
+            case PGA_MUTATION_DE:
+                switch (ctx->ga.DECrossoverType) {
+                case PGA_DE_CROSSOVER_BIN:
+                    do_crossover =
+                        (  idx == midx
+                        || PGARandomFlip (ctx, ctx->ga.DECrossoverProb)
+                        );
+                    break;
+                case PGA_DE_CROSSOVER_EXP:
+                    /* The first index copied is midx, then all indices
+                     * are copied while the coin flip is valid
+                     */
+                    if (do_crossover) {
+                        idx = (midx + i) % ctx->ga.StringLen;
+                        if (i > 0) {
+                            do_crossover =
+                                (PGARandomFlip (ctx, ctx->ga.DECrossoverProb));
+                        }
+                    }
+                    break;
+                default:
+                    PGAError
+                        ( ctx, "PGARealMutation: Invalid DE crossover type:"
+                        , PGA_FATAL, PGA_INT
+                        , (void *) &(ctx->ga.DECrossoverType)
+                        );
+                    break;
+                }
+                if (do_crossover){
+                    double f = ctx->ga.DEScaleFactor + de_dither;
+                    if (ctx->ga.DEJitter > 0) {
+                        f += ctx->ga.DEJitter * (PGARandom01 (ctx, 0) - 0.5);
+                    }
+                    switch (ctx->ga.DEVariant) {
+                    case PGA_DE_VARIANT_RAND:
+                    case PGA_DE_VARIANT_BEST:
+                        /* the last element is either a random individual
+                         * or the best depending on variant
+                         */
+                        c [idx] = indivs [maxidx - 1][idx];
+                        /* Add difference vectors */
+                        for (j=0; j < (maxidx - 1); j+=2) {
+                            c [idx] +=
+                                f * (indivs [j][idx] - indivs [j+1][idx]);
+                        }
+                        break;
+                    case PGA_DE_VARIANT_EITHER_OR:
+                        /* We use only 1 difference and ignore DENumDiffs */
+                        if (PGARandom01 (ctx, 0) < ctx->ga.DEProbabilityEO) {
+                            c [idx] = indivs [0][idx]
+                                  + f * (indivs [1][idx] - indivs [2][idx]);
+                        } else {
+                            double k = ctx->ga.DEAuxFactor;
+                            c [idx] = indivs [0][idx]
+                                  + k * ( indivs [1][idx] + indivs [2][idx]
+                                        - 2 * indivs [0][idx]
+                                        );
+                        }
+                        break;
+                    default:
+                        PGAError(ctx, "PGARealMutation: Invalid value of "
+                                 "ga.DEVariant:", PGA_FATAL, PGA_INT,
+                                 (void *) &(ctx->ga.DEVariant));
+                        break;
+                    }
+                    count++;
+                }
+                break;
+            default:
+                PGAError(ctx, "PGARealMutation: Invalid value of "
+                         "ga.MutationType:", PGA_FATAL, PGA_INT,
+                         (void *) &(ctx->ga.MutationType));
+                break;
+        }
 
-     PGADebugExited("PGARealMutation");
+        /* reset to min/max or bounce if outside range */
+        if (ctx->ga.MutateBoundedFlag || ctx->ga.MutateBounceFlag) {
+            if( c [idx] < ctx->init.RealMin [idx]) {
+                if (ctx->ga.MutateBounceFlag) {
+                    c [idx] = PGARandomUniform
+                        (ctx, ctx->init.RealMin [idx], old_value);
+                } else {
+                    c [idx] = ctx->init.RealMin [idx];
+                }
+            }
+            if( c [idx] > ctx->init.RealMax [idx]) {
+                if (ctx->ga.MutateBounceFlag) {
+                    c [idx] = PGARandomUniform
+                        (ctx, old_value, ctx->init.RealMax [idx]);
+                } else {
+                    c [idx] = ctx->init.RealMax [idx];
+                }
+            }
+        }
+    }
 
-     return(count);
+    PGADebugExited("PGARealMutation");
+
+    return(count);
 }
 
 /*I****************************************************************************
@@ -814,19 +1006,21 @@ void PGARealCopyString ( PGAContext *ctx, int p1, int pop1, int p2, int pop2)
 ****************************************************************************I*/
 int PGARealDuplicate( PGAContext *ctx, int p1, int pop1, int p2, int pop2)
 {
-     PGAReal *a = (PGAReal *)PGAGetIndividual(ctx, p1, pop1)->chrom;
-     PGAReal *b = (PGAReal *)PGAGetIndividual(ctx, p2, pop2)->chrom;
-     int i;
+    PGAReal *a = (PGAReal *)PGAGetIndividual (ctx, p1, pop1)->chrom;
+    PGAReal *b = (PGAReal *)PGAGetIndividual (ctx, p2, pop2)->chrom;
+    int count = 0;
 
-    PGADebugEntered("PGARealDuplicate");
+    PGADebugEntered ("PGARealDuplicate");
 
-     i = ctx->ga.StringLen-1;
-     if (a[0] == b[0])
-       for(; (i>0) && (a[i] == b[i]); i--);
+    for (count=0; count<ctx->ga.StringLen; count++) {
+        if (a [count] != b [count]) {
+            break;
+        }
+    }
 
-    PGADebugExited("PGARealDuplicate");
+    PGADebugExited ("PGARealDuplicate");
 
-     return((i==0) ? PGA_TRUE : PGA_FALSE);
+    return count == ctx->ga.StringLen ? PGA_TRUE : PGA_FALSE;
 }
 
 /*I****************************************************************************
@@ -882,41 +1076,56 @@ void PGARealInitString ( PGAContext *ctx, int p, int pop)
 ****************************************************************************I*/
 MPI_Datatype PGARealBuildDatatype(PGAContext *ctx, int p, int pop)
 {
-
-     int            counts[4];      /* Number of elements in each
-                                       block (array of integer) */
-     MPI_Aint       displs[4];      /* byte displacement of each
-                                       block (array of integer) */
-     MPI_Datatype   types[4];       /* type of elements in each block (array
-                                       of handles to datatype objects) */
-     MPI_Datatype   individualtype; /* new datatype (handle) */
-     PGAIndividual *traveller;      /* address of individual in question */
+    int            n = 6;
+    int            counts[7];      /* Number of elements in each
+                                      block (array of integer) */
+    MPI_Aint       displs[7];      /* byte displacement of each
+                                      block (array of integer) */
+    MPI_Datatype   types[7];       /* type of elements in each block (array
+                                      of handles to datatype objects) */
+    MPI_Datatype   individualtype; /* new datatype (handle) */
+    PGAIndividual *traveller;      /* address of individual in question */
 
     PGADebugEntered("PGARealBuildDatatype");
 
-     traveller = PGAGetIndividual(ctx, p, pop);
-     MPI_Get_address(&traveller->evalfunc, &displs[0]);
-     counts[0] = 1;
-     types[0]  = MPI_DOUBLE;
+    traveller = PGAGetIndividual(ctx, p, pop);
+    MPI_Get_address(&traveller->evalue, &displs[0]);
+    counts[0] = 1;
+    types[0]  = MPI_DOUBLE;
 
-     MPI_Get_address(&traveller->fitness, &displs[1]);
-     counts[1] = 1;
-     types[1]  = MPI_DOUBLE;
+    MPI_Get_address(&traveller->fitness, &displs[1]);
+    counts[1] = 1;
+    types[1]  = MPI_DOUBLE;
 
-     MPI_Get_address(&traveller->evaluptodate, &displs[2]);
-     counts[2] = 1;
-     types[2]  = MPI_INT;
+    MPI_Get_address(&traveller->evaluptodate, &displs[2]);
+    counts[2] = 1;
+    types[2]  = MPI_INT;
 
-     MPI_Get_address(traveller->chrom, &displs[3]);
-     counts[3] = ctx->ga.StringLen;
-     types[3]  = MPI_DOUBLE;
+    MPI_Get_address(traveller->chrom, &displs[3]);
+    counts[3] = ctx->ga.StringLen;
+    types[3]  = MPI_DOUBLE;
 
-     MPI_Type_create_struct(4, counts, displs, types, &individualtype);
-     MPI_Type_commit(&individualtype);
+    MPI_Get_address(&traveller->auxtotal, &displs[4]);
+    counts[4] = 1;
+    types[4]  = MPI_DOUBLE;
+
+    MPI_Get_address(&traveller->auxtotalok, &displs[5]);
+    counts[5] = 1;
+    types[5]  = MPI_INT;
+
+    if (ctx->ga.NumAuxEval) {
+        MPI_Get_address(traveller->auxeval, &displs[6]);
+        counts[6] = ctx->ga.NumAuxEval;
+        types[6]  = MPI_DOUBLE;
+        n += 1;
+    }
+
+    MPI_Type_create_struct(n, counts, displs, types, &individualtype);
+    MPI_Type_commit(&individualtype);
 
     PGADebugExited("PGARealBuildDatatype");
 
-     return (individualtype);
+    return (individualtype);
 }
 
 /*I****************************************************************************
@@ -952,3 +1161,36 @@ double PGARealGeneDistance (PGAContext *ctx, int p1, int pop1, int p2, int pop2)
     return ret;
 }
 
+/*I****************************************************************************
+   PGARealEuclidianDistance - Compute genetic difference of two strings.
+   This uses the Euclidian distance metric, the square-root of the sum
+   of all squared differences of each allele.
+
+   Inputs:
+      ctx   - context variable
+      p1    - first string index
+      pop1  - symbolic constant of the population the first string is in
+      p2    - second string index
+      pop2  - symbolic constant of the population the second string is in
+
+   Outputs:
+      genetic euclidian distance of the two strings
+
+   Example:
+      Use in PGASetUserFunction.
+
+****************************************************************************I*/
+double PGARealEuclidianDistance (PGAContext *ctx, int p1, int pop1, int p2, int pop2)
+{
+    PGAReal *c1 = (PGAReal *)PGAGetIndividual (ctx, p1, pop1)->chrom;
+    PGAReal *c2 = (PGAReal *)PGAGetIndividual (ctx, p2, pop2)->chrom;
+    double ret = 0.0;
+    int i;
+
+    PGADebugEntered("PGARealGeneDistance");
+    for (i=0; i<ctx->ga.StringLen; i++) {
+        ret += pow (c1 [i] - c2 [i], 2);
+    }
+    PGADebugExited("PGARealGeneDistance");
+    return sqrt (ret);
+}
