@@ -47,16 +47,83 @@ privately owned rights.
 
 #include "pgapack.h"
 
+/*U****************************************************************************
+  INDGetAuxTotal - Compute total value over all aux evaluations
+
+  Category: Operators
+
+  Inputs:
+    ind   - Pointer to Individual
+
+  Outputs:
+    Computed or cached total value over all aux evaluations
+    This is the sum of all *positive* individual aux evaluations.
+    The semantics is a total value of all constraint violations.
+
+  Example:
+    PGAIndividual *ind = PGAGetIndividual (ctx, p, PGA_OLDPOP);
+    double result;
+    :
+    result = INDGetAuxTotal (ind);
+
+****************************************************************************U*/
+double INDGetAuxTotal (PGAIndividual *ind)
+{
+    PGAContext *ctx = ind->ctx;
+    if (!ind->auxtotalok) {
+        int i;
+        double s = 0;
+        int numaux = ctx->ga.NumAuxEval;
+        int numcon = ctx->ga.NumConstraint;
+        for (i=numaux - numcon; i<numaux; i++) {
+            if (ind->auxeval [i] > 0) {
+                s += ind->auxeval [i];
+            }
+        }
+        ind->auxtotal   = s;
+        ind->auxtotalok = PGA_TRUE;
+    }
+    return ind->auxtotal;
+}
+
+/*U****************************************************************************
+  PGAGetAuxTotal - Compute total value over all aux evaluations
+
+  Category: Operators
+
+  Inputs:
+    ctx   - context variable
+    p     - individual
+    pop   - population
+
+  Outputs:
+    Computed or cached total value over all aux evaluations
+    This is the sum of all *positive* individual aux evaluations.
+    The semantics is a total value of all constraint violations.
+
+  Example:
+    PGAContext *ctx;
+    double result;
+    :
+    result = PGAGetAuxTotal(ctx, p, PGA_OLDPOP);
+
+****************************************************************************U*/
+double PGAGetAuxTotal (PGAContext *ctx, int p, int pop)
+{
+    PGAIndividual *ind = PGAGetIndividual(ctx, p, pop);
+    return INDGetAuxTotal (ind);
+}
+
 /*U****************************************************************************
   PGASelect - performs genetic algorithm selection using either the default
   selection scheme or that specified with PGASetSelectType().  Valid selection
   methods are proportional, stochastic universal, tournament, probabilistic
-  tournament selection, or truncation selection,
+  tournament selection, truncation selection, or linear selection,
   PGA_SELECT_PROPORTIONAL, PGA_SELECT_SUS, PGA_SELECT_TOURNAMENT,
-  PGA_SELECT_PTOURNAMENT, PGA_SELECT_TRUNCATION, respectively.  This
-  function updates an internal array with the indices of members of popix
-  selected for recombination.  These indices may be accessed with
-  PGASelectNextIndex()
+  PGA_SELECT_PTOURNAMENT, PGA_SELECT_TRUNCATION, PGA_SELECT_LINEAR
+  respectively. This function updates an internal array with the
+  indices of members of popix selected for recombination.  These indices
+  may be accessed with PGASelectNextIndex()
 
   Category: Operators
 
@@ -96,15 +163,19 @@ void PGASelect (PGAContext *ctx, int popix)
         break;
     case PGA_SELECT_TOURNAMENT:    /* tournament selection               */
         for (i=0; i<ctx->ga.PopSize; i++)
-            ctx->ga.selected[i] = PGASelectTournament (ctx, pop);
+            ctx->ga.selected[i] = PGASelectTournament (ctx, popix);
         break;
     case PGA_SELECT_PTOURNAMENT:   /* probabilistic tournament selection */
         for (i=0; i<ctx->ga.PopSize; i++)
-            ctx->ga.selected[i] = PGASelectPTournament (ctx, pop);
+            ctx->ga.selected[i] = PGASelectPTournament (ctx, popix);
         break;
     case PGA_SELECT_TRUNCATION:   /* truncation selection */
         for (i=0; i<ctx->ga.PopSize; i++)
-            ctx->ga.selected[i] = PGASelectTruncation (ctx, pop);
+            ctx->ga.selected[i] = PGASelectTruncation (ctx, popix);
+        break;
+    case PGA_SELECT_LINEAR:      /* linear selection */
+        for (i=0; i<ctx->ga.PopSize; i++)
+            ctx->ga.selected[i] = PGASelectLinear (ctx, pop);
         break;
     default:
         PGAError( ctx,
@@ -119,9 +190,16 @@ void PGASelect (PGAContext *ctx, int popix)
      * Note that for all selection schemes above *except* SUS the items
      * are already randomized. So we randomize only if the selection
      * scheme is SUS *or* we have the backward-compatibility flag
-     * ctx->ga.RandomizeSelect set.
+     * ctx->ga.RandomizeSelect set. Note that the point of linear
+     * selection is that the sequence is *not* randomized. So in this
+     * case we never randomize the sequence.
      */
-    if (ctx->ga.SelectType == PGA_SELECT_SUS || ctx->ga.RandomizeSelect) {
+    if  (  ctx->ga.SelectType == PGA_SELECT_SUS
+        || (  ctx->ga.RandomizeSelect
+           && ctx->ga.SelectType != PGA_SELECT_LINEAR
+           )
+        )
+    {
         for (i=0; i<ctx->ga.PopSize; i++) {
             j          = PGARandomInterval(ctx, 0,ctx->ga.PopSize-1);
             temp       = ctx->ga.selected[j];
@@ -202,6 +280,7 @@ void PGASetSelectType( PGAContext *ctx, int select_type)
         case PGA_SELECT_TOURNAMENT:
         case PGA_SELECT_PTOURNAMENT:
         case PGA_SELECT_TRUNCATION:
+        case PGA_SELECT_LINEAR:
             ctx->ga.SelectType = select_type;
             break;
         default:
@@ -632,7 +711,7 @@ void PGASelectSUS( PGAContext *ctx, PGAIndividual *pop )
 
 /*I****************************************************************************
   PGASelectTournamentWithReplacement - chooses N strings randomly and
-  returns the one with highest fitness, N is the value set with
+  returns the one with best evaluation, N is the value set with
   PGASetTournamentSize, the default is 2. The selection happens *with*
   replacement. See decription of PGASelectTournamentWithoutReplacement
   for details of sampling without replacement.
@@ -643,8 +722,8 @@ void PGASelectSUS( PGAContext *ctx, PGAIndividual *pop )
           Genetic Algorithms (FOGA) 1, pp. 69-93, 1991.
 
   Inputs:
-    ctx   - context variable
-    popix - symbolic constant of population to select from
+    ctx - context variable
+    pop - symbolic constant of population to select from
 
   Outputs:
     index of the selected string
@@ -657,21 +736,15 @@ void PGASelectSUS( PGAContext *ctx, PGAIndividual *pop )
 
 ****************************************************************************I*/
 static
-int PGASelectTournamentWithReplacement( PGAContext *ctx, PGAIndividual *pop )
+int PGASelectTournamentWithReplacement (PGAContext *ctx, int pop)
 {
-    int m;
-    double maxfit;
     int i;
-
-    m = PGARandomInterval(ctx, 0, ctx->ga.PopSize-1);
-    maxfit = (pop+m)->fitness;
+    int m = PGARandomInterval (ctx, 0, ctx->ga.PopSize-1);
     for (i=1; i<ctx->ga.TournamentSize; i++) {
         int mn = PGARandomInterval(ctx, 0, ctx->ga.PopSize-1);
-        double fit = (pop+mn)->fitness;
         /* use '>=' for backwards-compat with prev. binary tournament */
-        if (fit >= maxfit) {
+        if (PGAEvalCompare (ctx, mn, pop, m, pop) <= 0) {
             m = mn;
-            maxfit = fit;
         }
     }
     return m;
@@ -679,7 +752,7 @@ int PGASelectTournamentWithReplacement( PGAContext *ctx, PGAIndividual *pop )
 
 /*I****************************************************************************
   PGASelectTournamentWithoutReplacement - chooses N strings randomly and
-  returns the one with highest fitness, N is the value set with
+  returns the one with best evaluation, N is the value set with
   PGASetTournamentSize, the default is 2. The selection happens *without*
   replacement. This means if we select N individuals with a tournament
   size of 2, each individual is participating in exactly two
@@ -691,8 +764,8 @@ int PGASelectTournamentWithReplacement( PGAContext *ctx, PGAIndividual *pop )
           Complex Systems, 3(5):493–530, 1989.
 
   Inputs:
-    ctx   - context variable
-    popix - symbolic constant of population to select from
+    ctx - context variable
+    pop - symbolic constant of population to select from
 
   Outputs:
     index of the selected string
@@ -714,7 +787,7 @@ static void _shuffle (PGAContext *ctx, int *list, int n)
     int i = 0, j = 0, tmp = 0;
     for (i = 0; i < n; i++)
 	list[i] = i;
-    for (i = 0; i < n-2; i++) {
+    for (i = 0; i < n-1; i++) {
 	j = PGARandomInterval (ctx, i, n - 1);
         tmp = list[j];
         list [j] = list [i];
@@ -727,10 +800,9 @@ static void _shuffle (PGAContext *ctx, int *list, int n)
       : (perm) [(idx)++]
 
 static
-int PGASelectTournamentWithoutReplacement (PGAContext *ctx, PGAIndividual *pop)
+int PGASelectTournamentWithoutReplacement (PGAContext *ctx, int pop)
 {
     int m;
-    double maxfit;
     int i;
     static int *permutation = NULL;
     static int perm_idx = 0;
@@ -746,22 +818,20 @@ int PGASelectTournamentWithoutReplacement (PGAContext *ctx, PGAIndividual *pop)
     }
 
     m = NEXT_IDX(ctx, permutation, perm_idx, ctx->ga.PopSize);
-    maxfit = (pop+m)->fitness;
     for (i=1; i<ctx->ga.TournamentSize; i++) {
         int mn = NEXT_IDX(ctx, permutation, perm_idx, ctx->ga.PopSize);
-        double fit = (pop+mn)->fitness;
-        if (fit >= maxfit) {
+        if (PGAEvalCompare (ctx, mn, pop, m, pop) <= 0) {
             m = mn;
-            maxfit = fit;
         }
     }
     return m;
 }
 
 /*I****************************************************************************
-  PGASelectTruncation - chooses the best k strings and returns them in
-  random order. The value k is (N * TruncationProportion) rounded to the
-  next integer.
+  PGASelectLinear - chooses all strings that are not already copied to
+  the next generation due to elitist strategies. Note that this
+  'selection' scheme isn't a selection scheme in the genetic sense, it
+  has no selection pressure. Note that the indeces are *not randomized.
 
   Inputs:
     ctx   - context variable
@@ -770,6 +840,39 @@ int PGASelectTournamentWithoutReplacement (PGAContext *ctx, PGAIndividual *pop)
   Outputs:
     index of the selected string
 
+  Example:
+    PGAContext *ctx,
+    int l;
+    :
+    l = PGASelectLinear (ctx, PGA_OLDPOP);
+
+****************************************************************************I*/
+int PGASelectLinear (PGAContext *ctx, PGAIndividual *pop)
+{
+    static int perm_idx = 0;
+    static int last_generation = -1;
+    int numreplace = PGAGetNumReplaceValue (ctx);
+    int popsize = PGAGetPopSize (ctx);
+
+    if (last_generation != ctx->ga.iter || perm_idx >= popsize) {
+        perm_idx = popsize - numreplace;
+        last_generation = ctx->ga.iter;
+    }
+    return perm_idx++;
+}
+
+/*I****************************************************************************
+  PGASelectTruncation - chooses the best k strings and returns them in
+  random order. The value k is (N * TruncationProportion) rounded to the
+  next integer.
+
+  Inputs:
+    ctx - context variable
+    pop - symbolic constant of population to select from
+
+  Outputs:
+    index of the selected string
+
   Example:
     PGAContext *ctx,
     int l;
@@ -777,7 +880,7 @@ int PGASelectTournamentWithoutReplacement (PGAContext *ctx, PGAIndividual *pop)
     l = PGASelectTruncation (ctx, PGA_OLDPOP);
 
 ****************************************************************************I*/
-int PGASelectTruncation (PGAContext *ctx, PGAIndividual *pop)
+int PGASelectTruncation (PGAContext *ctx, int pop)
 {
     int m = -1;
     int k = (int)(ctx->ga.PopSize * ctx->ga.TruncProportion + 0.5);
@@ -802,13 +905,11 @@ int PGASelectTruncation (PGAContext *ctx, PGAIndividual *pop)
         perm_idx = k;
     }
     if (last_generation != ctx->ga.iter) {
-        int bestidx [ctx->ga.PopSize];
         int i;
-        for (i=0; i<ctx->ga.PopSize; i++) {
-            bestidx [i] = i;
-            ctx->scratch.dblscratch [i] = pop [i].fitness;
-        }
-        PGADblHeapSort (ctx, ctx->scratch.dblscratch, bestidx, ctx->ga.PopSize);
+        int bestidx [ctx->ga.PopSize];
+        /* Returns sorted list of indeces in bestidx, no need to initialize */
+        PGAEvalSort (ctx, pop, bestidx);
+        /* Copy the k first indeces */
         for (i=0; i<k; i++) {
             kbest [i] = bestidx [i];
         }
@@ -822,14 +923,14 @@ int PGASelectTruncation (PGAContext *ctx, PGAIndividual *pop)
 
 /*I****************************************************************************
   PGASelectTournament - chooses N strings randomly and
-  returns the one with highest fitness, N is the value set with
+  returns the one with best evaluation, N is the value set with
   PGASetTournamentSize, the default is 2. Depending on the setting of
   PGASetTournamentWithReplacement calls one of two local functions to
   use the right sampling.
 
   Inputs:
-    ctx   - context variable
-    popix - symbolic constant of population to select from
+    ctx - context variable
+    pop - symbolic constant of population to select from
 
   Outputs:
     index of the selected string
@@ -841,7 +942,7 @@ int PGASelectTruncation (PGAContext *ctx, PGAIndividual *pop)
     l = PGASelectTournament(ctx, PGA_OLDPOP);
 
 ****************************************************************************I*/
-int PGASelectTournament (PGAContext *ctx, PGAIndividual *pop)
+int PGASelectTournament (PGAContext *ctx, int pop)
 {
     PGADebugEntered("PGASelectTournament");
     if (ctx->ga.TournamentWithRepl) {
@@ -854,12 +955,12 @@ int PGASelectTournament (PGAContext *ctx, PGAIndividual *pop)
 
 /*I****************************************************************************
   PGASelectPTournament - chooses two strings randomly and returns the one with
-  higher fitness with a specified probability
+  better evaluation with a specified probability
   Ref:    D. Goldberg, Genetic Algorithms, pg. 121
 
   Inputs:
-    ctx   - context variable
-    popix - symbolic constant of population to select from
+    ctx - context variable
+    pop - symbolic constant of population to select from
 
   Outputs:
     index of the selected string
@@ -871,29 +972,24 @@ int PGASelectTournament (PGAContext *ctx, PGAIndividual *pop)
     l = PGASelectPTournament(ctx, PGA_OLDPOP);
 
 ****************************************************************************I*/
-int PGASelectPTournament( PGAContext *ctx, PGAIndividual *pop )
+int PGASelectPTournament (PGAContext *ctx, int pop)
 {
     int m1, m2;
     int RetVal;
+    double drand;
 
     PGADebugEntered("PGASelectPTournament");
 
     m1 = PGARandomInterval(ctx, 0, ctx->ga.PopSize-1);
     m2 = PGARandomInterval(ctx, 0, ctx->ga.PopSize-1);
+    drand = PGARandom01 (ctx, 0);
 
-    if ( (pop+m1)->fitness > (pop+m2)->fitness )
-        if ( (double) PGARandom01(ctx, 0) < ctx->ga.PTournamentProb )
-            RetVal = m1;
-        else
-            RetVal = m2;
-    else
-        if ( (double) PGARandom01(ctx, 0) < ctx->ga.PTournamentProb )
-            RetVal = m2;
-        else
-            RetVal = m1;
+    if (PGAEvalCompare (ctx, m1, pop, m2, pop) < 0) {
+        RetVal = drand < ctx->ga.PTournamentProb ? m1 : m2;
+    } else {
+        RetVal = drand < ctx->ga.PTournamentProb ? m2 : m1;
+    }
 
     PGADebugExited("PGASelectPTournament");
     return(RetVal);
 }
-
-
